#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <fitsio.h>

#define KBOLTZ 8.617e-8
#define MAXSTRLEN 1024
#define BITPIX 8
#define NCOLUMNS 10

void fits_spec_write_cocorow(fitsfile *fptr, int iRow, int Z, int rmJ, 
			     int Max_Real, int Max_Pseudo, 
			     int N_Real, double *E_Real, double *Real,
			     int N_Pseudo, double *E_Pseudo, double *Pseudo);

void fits_printerror(char *process, int status);
char *make_tform(int value, char type, char *result);

void fits_spec_write_cocoHDU(int N_Z, int list_Z[], int list_rmJ[],
			     fitsfile *fptr, double T, double Ne, 
			     double tot_emission, 
			     double **E_real, double **E_pseudo, 
			     double **real_cont, double **pseudo_cont,
			     int *Ncomp_real, int *Ncomp_pseudo) {
  
  int tfields;
  char hduname[20];
  char *ttype[NCOLUMNS], *tunit[NCOLUMNS], *tform[NCOLUMNS];
  float kT_fl, T_fl, ne_fl, tot_emis_fl;
  int i, iRow, status=0;
  long Nrow, firstrow;
  int extver, chdu;
  int Nmax_real=0, Nmax_pseudo=0;
  int iZ;

  Nrow = N_Z;

  /* Stage 1: Figure out what the maximum size of the column vectors is */
  for (iZ=0;iZ<N_Z;iZ++) {
    if (Ncomp_real[iZ] > Nmax_real) Nmax_real = Ncomp_real[iZ];
    if (Ncomp_pseudo[iZ] > Nmax_pseudo) Nmax_pseudo = Ncomp_pseudo[iZ];
  }

  kT_fl= (float) KBOLTZ*T;
  T_fl = (float) T;
  ne_fl= (float) Ne;
  tot_emis_fl = (float) tot_emission;
    
  for(i=0;i<NCOLUMNS;i++) {
    ttype[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
    tunit[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
    tform[i] = (char *) malloc(MAXSTRLEN*sizeof(char));
  }
  
  tfields = NCOLUMNS;
  sprintf(ttype[0],"Z");
  sprintf(tform[0],"1J");
  sprintf(tunit[0]," ");
  sprintf(ttype[1],"rmJ");
  sprintf(tform[1],"1J");
  sprintf(tunit[1]," ");

  sprintf(ttype[2],"N_Cont");
  sprintf(tunit[2]," ");
  sprintf(tform[2],"1J");
  sprintf(ttype[3],"E_Cont");
  sprintf(tunit[3],"keV");
  make_tform(Nmax_real,'E',tform[3]);
  sprintf(ttype[4],"Continuum");
  sprintf(tunit[4],"photons cm^3 s^-1 keV^-1");
  make_tform(Nmax_real,'E',tform[4]);
  sprintf(ttype[5],"Cont_Err ");
  sprintf(tunit[5],"photons cm^3 s^-1 keV^-1");
  make_tform(Nmax_real,'E',tform[5]);

  sprintf(ttype[6],"N_Pseudo");
  sprintf(tunit[6]," ");
  sprintf(tform[6],"1J");
  sprintf(ttype[7],"E_Pseudo");
  sprintf(tunit[7],"keV");
  make_tform(Nmax_pseudo,'E',tform[7]);
  sprintf(ttype[8],"Pseudo");
  sprintf(tunit[8],"photons cm^3 s^-1 keV^-1 ");
  make_tform(Nmax_pseudo,'E',tform[8]);
  sprintf(ttype[9],"Pseudo_Err");
  sprintf(tunit[9],"photons cm^3 s^-1 keV^-1 ");
  make_tform(Nmax_pseudo,'E',tform[9]);
  
  if (Ne > 0) {
    sprintf(hduname,"C%.1f_%.1f",log10(T),log10(Ne));
  } else {
    sprintf(hduname,"C%.1f",log10(T));
  }    

  if (fits_create_tbl(fptr, BINARY_TBL, 0, tfields, ttype, tform,
		      tunit, "EMISSIVITY", &status)) {
    fits_printerror("writing main header keywords",status);
  }

  fits_get_hdu_num(fptr, &extver);
  extver = extver-2; /* Don't count PRIMARY or PARAMETER HDU; start at 1 */

  if (fits_write_key(fptr, TINT, "EXTVER", &extver,
		     "Index for this EMISSIVITY extension", &status)) {
    fits_printerror("writing EXTVER keyword",status);
  }
  
  if (fits_write_key(fptr, TSTRING, "HDUNAME", hduname,
		     "Spectral emission data", &status)) {
    fits_printerror("writing HDUCLASS keyword",status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLASS", "Proposed OGIP",
		     "Proposed OGIP standard", &status)) {
    fits_printerror("writing HDUCLASS keyword",status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLAS1", "COMP CONT MODEL",
		     "Compressed continua spectra", &status)) {
    fits_printerror("writing HDUCLAS1 keyword", status);
  }

  if (fits_write_key(fptr, TSTRING, "HDUCLAS2", "COCO",
		     "Compressed continuum data", &status)) {
    fits_printerror("writing HDUCLAS2 keyword", status);
  }
  
  if (fits_write_key(fptr, TSTRING, "HDUVERS1", "1.0.0",
		     "version of format", &status)) {
    fits_printerror("writing HDUVERS1 keyword", status);
  }

  /* Write the parameters describing the table */
  
  if (fits_write_key(fptr, TFLOAT, "TEMPERATURE", &T_fl,"Electron temperature",
		     &status)) {
    fits_printerror("writing TEMPERATURE keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "DENSITY", &ne_fl,"Electron density",
		     &status)) {
    fits_printerror("writing DENSITY keyword",status);
  }

  if (fits_write_key(fptr, TFLOAT, "TOT_COCO", &tot_emis_fl,
		     "Total Emission (erg cm^3 s^-1)", &status)) {
    fits_printerror("writing TOT_COCO keyword",status); 
  }

  /* Now we're ready to write out the rows that go into this file. */
  /* The N rows will be for the summed emission over each atom calculated. */

  /* First row */
  iRow = 1;

  for (iZ=0;iZ<N_Z;iZ++) {
    if (list_Z == NULL) { /* Total spectrum only */
      fits_spec_write_cocorow(fptr, iRow, 0, 0, Nmax_real, Nmax_pseudo, 
			      Ncomp_real[iZ], E_real[iZ], real_cont[iZ],
			      Ncomp_pseudo[iZ], E_pseudo[iZ], pseudo_cont[iZ]);
    } else if (list_rmJ == NULL) { /* Per element */
      fits_spec_write_cocorow(fptr, iRow, list_Z[iZ], 0, Nmax_real, 
			      Nmax_pseudo, Ncomp_real[iZ], E_real[iZ], 
			      real_cont[iZ], Ncomp_pseudo[iZ], E_pseudo[iZ], 
			      pseudo_cont[iZ]);
    } else { /* Per ion */
      fits_spec_write_cocorow(fptr, iRow, list_Z[iZ], list_rmJ[iZ], Nmax_real,
			      Nmax_pseudo, Ncomp_real[iZ], E_real[iZ], 
			      real_cont[iZ], Ncomp_pseudo[iZ], E_pseudo[iZ], 
			      pseudo_cont[iZ]);
    }
    iRow++;
  }

  for(i=0;i<NCOLUMNS;i++) {
    free(ttype[i]);
    free(tunit[i]);
    free(tform[i]);
  }

  /* Now, jump back to the first BINARY HDU and write the necessary info. */
  /* Start by saving the current HDU so we can return to it */

  fits_get_hdu_num(fptr, &chdu);
  
  /* Jump to the PARAMETERS hdu */
  if (fits_movabs_hdu(fptr, 2, NULL, &status)) {
    fits_printerror("moving to PARAMETERS hdu.",status);
  }
  
  /* Write out the relevant row data; row number is, conveniently, = extver. */
  firstrow = (long) extver;
  
  if (fits_write_col(fptr, TFLOAT, 1, firstrow, 1, 1, &kT_fl, &status)) {
    fits_printerror("writing temperature, in keV",status);
  }

  if (fits_write_col(fptr, TFLOAT, 2, firstrow, 1, 1, &ne_fl, &status)) {
    fits_printerror("writing electron density, in cm^-3",status);
  }
  
  if (fits_write_col(fptr, TINT, 3, firstrow, 1, 1, &N_Z, &status)) {
    fits_printerror("writing number of elements",status);
  }

  if (fits_write_col(fptr, TINT, 4, firstrow, 1, 1, &Nmax_real, &status)) {
    fits_printerror("writing number of real continuum points",status);
  }

  if (fits_write_col(fptr, TINT, 5, firstrow, 1, 1, &Nmax_pseudo, &status)) {
    fits_printerror("writing number of pseudo-continuum points",status);
  }

  /* And now jump back to the most recent HDU */
  if (fits_movabs_hdu(fptr, chdu, NULL, &status)) {
    fits_printerror("returning to current HDU.",status);
  }

}
