#include <stdlib.h>
#include <stdio.h>

#ifdef USE_PFILE
#include <pfile.h>
#endif

#include "apec.h" 
#include "messages.h"
#include "fitsio.h"

/* function definitions in this file */
#include "apec_funcdefs.h"

int main(int argc, char *argv[]) {

  struct ELEMENT_ION_STATE *state;
  ElementP *Atomic_params;
  struct DEBUG_FILES *debug;
  struct PARAMETERS *params;
  struct LEVEL_RATES level_rates[MAXIMUM_Z+1];
  struct TRANS_RATES *trans_rates; 
  struct EMISSION_LINES *line=NULL, *ionrec_line=NULL;
  struct BIN_SPEC *spectrum;
  struct TWOPHOT_DATA *twophot;
  struct ION_BAL_TABLE ionbal;
  double *abund;
  int iEl, irmJ, iTemp, iDens, nlev;
  int Z, rmJ;
  int maxZ, maxNlev;
  double Te, Ne;
  double Ion_pop[MAXIMUM_Z+1]; /* These relate to the AR data */
  double *Lev_pop=NULL;        /* level population */

  /* Driver for apec library, does calculations as set up in the
     parameter file loaded below. */

  debug       = (struct DEBUG_FILES *) malloc(sizeof(struct DEBUG_FILES));
  params      = (struct PARAMETERS *) malloc(sizeof(struct PARAMETERS));
  trans_rates = (struct TRANS_RATES *) malloc(sizeof(struct TRANS_RATES));
  spectrum    = (struct BIN_SPEC *) malloc(sizeof(struct BIN_SPEC));
  twophot     = (struct TWOPHOT_DATA *) malloc(sizeof(struct TWOPHOT_DATA));

  get_input_data(argc, argv, NULL, params, debug);

  /* Calls apec library initialization steps. */
  initialize(params, spectrum, &Atomic_params, &abund, &ionbal, 
	     trans_rates, &Lev_pop, debug, &maxZ, &maxNlev);

  for (iDens = 0; iDens < params->Ndens; iDens++) {
    Ne = params->density[iDens];

    for (iTemp=0; iTemp < params->Ntemp ; iTemp++) {
      Te = params->temperature[iTemp];
      
      message("apec","T = %e, n = %e",Te, Ne);

      for (iEl = 0; iEl < params->N_Elements; iEl++) {

	state = Atomic_params[iEl];
	Z = params->list_Z[iEl];

	/* Print out the numerical values in structure arrays */
	print_table(debug, Atomic_params[iEl]);

	/* ********************************************************* */
	/*    Calculate ion population for ions of interest          */
	/* ********************************************************* */

	if (params->use_ionbal_table) { /* Load table-driven ion bal */
	  find_ion_bal(Z, Te, Ne, Ion_pop, &ionbal, params);
	
	  for (rmJ = 1; rmJ < Z+1; rmJ++) {
	    
	    allocate_level_rates(state[rmJ-1].ion_inf->nlev,
				 &(level_rates[rmJ-1]));
	    level_rates[rmJ-1].qioniz_total = 0.0;
	    if (rmJ-1 < Z ) {
	      level_rates[rmJ-1].qrecomb_total = 0.0;
	      if (Ion_pop[rmJ-1] > 0.0) {
		level_rates[rmJ-1].ionc = Ion_pop[rmJ]/Ion_pop[rmJ-1];
	      } else {
		level_rates[rmJ-1].ionc = Ion_pop[rmJ]*BLOWUP;
	      }
	    }
	  }
	} else { 
	  calc_ionbal(debug, state, NULL, Te, Ne, 0.0, Ion_pop, &ionrec_line);
	}

	/* ********************************************************* */
	/*          Calculate level population and emission          */
	/* ********************************************************* */

	for (irmJ=0; irmJ < params->list_NrmJ[iEl]; irmJ++) {
	  /* Set up the spectrum arrays for all ions. */
	  alloc_spectrum(Z, params->list_rmJ[iEl][irmJ], spectrum);
	}

	for (irmJ=0; irmJ < params->list_NrmJ[iEl]; irmJ++) {

	  rmJ = params->list_rmJ[iEl][irmJ];
	  nlev=state[rmJ-1].ion_inf->nlev;
	  
	  if ((nlev > 0) && (Ion_pop[rmJ-1] > MIN_ION_POP)) {

	    if (params->do_rrc) {
	      calc_rad_rec_cont(debug, Z, rmJ, abund[Z], Ion_pop[rmJ],
				&(state[rmJ]), &(state[rmJ-1]), 
				Te, params->min_epsilon, 
				level_rates[rmJ-1].qrecomb, spectrum);
	    }
	    if (params->do_sat) {
	      calc_satellite(debug, abund[Z], Ion_pop[rmJ], 
			     &(state[rmJ]), &(state[rmJ-1]), 
			     Te, Ne, level_rates[rmJ-1].qsatel, &line);
	    }

	    if (params->do_lines) { /* Calculate line emission. */
	      calc_trans_rates(debug, &(state[rmJ-1]), Te, Te, Ne, 
			       trans_rates);

	    /* Set up rate matrix and inversion. Return Lev_pop with */
	    /* total pop of each ion normalized to one. */
	      solve_rate_eqns(debug, trans_rates, &(level_rates[rmJ-1]),
			      Ne, Lev_pop);
	      calc_emission(debug, abund, Ion_pop[rmJ-1], Lev_pop, 
			    &(state[rmJ-1]), Te, Ne, 
			    twophot, &line);
	      
	      calc_two_phot(debug, &(state[rmJ-1]), twophot, spectrum);

	      if ((state[rmJ-1].ion_inf)->nemiss != 0)
		calc_emission_features(debug, &(state[rmJ-1]), 
				       abund[Z], Te, Ion_pop[rmJ-1], &line, 
				       spectrum);
	      
	      bin_lines(Z, rmJ, params->min_epsilon, line, spectrum);
	    }
	  }

	  if ((params->do_brems)&&(Ion_pop[rmJ-1]>MIN_ION_POP)&&(rmJ>1)) {
	    bremsstrahlung(Z, rmJ, abund[Z], Ion_pop[rmJ-1], 
			   params->filemap_file, params->Brems_Type, 
			   Te, spectrum);
	  }
	}
	
	if (params->nei_output) 
	  calc_spectrum_per_ion(Z, Ion_pop, line, spectrum);

	dealloc_level_rates(Z, level_rates);
      }  /* Close the Element loop */

      fits_output(params, Te, Ne, iTemp, iDens, &line, spectrum);

      clear_spectrum(params->N_Elements, params->list_Z, spectrum);
      clear_lines(&line);
      
    } /* Close the Te loop */
  }   /* Close the Ne loop */
  
  /* Close all output files */
  close_debug_output(debug);
  fits_close(params);

  return(0);
}
