#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "messages.h"
#include "apec.h"

/*! \file calc_rrc.c */

/* Trial value */
#define TOT_ABSACC 1.e-21
#define TOT_RELACC 0.0001
/* Trial value */
#define BIN_ABSACC 1.e-21
#define BIN_RELACC 0.01

/*!< in erg cm / s / keV ; see below */
#define RRC_COEFF 1.31e8 

double dqags(double f(),double a,double b,double epsabs,
	     double epsrel,double *abserr,int *neval,int *ier);
double dqagi(double f(),double bound,int inf,double epsabs,
	     double epsrel,double *abserr,int *neval,int *ier);
char * roman_element(int Z,int rmJ);
double rrc_ph_value(double energy);
double rrc_erg_value(double energy);
double sigma_photoion(double energy);
double sigma_hydrogenic(int Z, int n, int l, double E);
void sigma_hydrogenic_arr(int Z, int n, int l, int Nbin, double bin_edge[], 
			  double sigma[]);
  /* \brief Calculate the radiative recombination rate and continuum to all levels 
   * \param Z proton number of element
   * \param abund Abundance of element, relative to hydrogen 
   * \param T Temperature, in Kelvin
   * \param ion_pop fractional population of the initial state ion 
   * \param I_e Ionization energy of ion, in keV
   * \param g_ratio Ratio of the statistical weight of the initial and final states.
   * \param sig_type Type of sigma data for the final state ion
   * \param sigma_coeff Array to use to calculate sigma, which must be in cm^2
   * \param spectrum Structure holding the resultant spectrum [optional]
   *
   * See notes on RRC in apec documentation. The sigma used is the 
   * photoionization cross section for the final state.  The idea is that 
   * recombination from an initial state to a final state is the inverse 
   * process from photoionization, and can therefore be directly linked.  
   * Therefore, the I_e and sigma values are for the _final_ state, while the 
   * ion_pop value is the population of the _initial_ state.
   * 
   * dP/dE = n_e n_z E_g sigma_{rec,n}(E_e) v_e f(v) dv / dE_g
   *       = (4 pi / c^2) (2 pi m_e k T)^(-3/2) E_g^3 sigma_{ph,n}(E_g)
   *         (g_z/g_{z+1}) exp(-(E_g - IonE)/kT) T^(-3/2)
   *       = (1.31 x 10^8 erg cm / s / keV) n_e n_z (E_g/keV)^3 
   *     sigma_{ph,n}(E_g (g_z/g_{z+1}) exp(-(E_g - IonE)/kT) (T/1 K)^(-3/2)
   *
   * Sigma approximations: 
   *  NO_PHOT: Default, sigma = 0 everywhere version
   *  VERNER: Verner & Yakovklev, A&AS 1995, 109, 125-133
   *     Complete for all ground states of all subshells. If E > E_th, then
   *     \sigma(E) = \sigma_0 F(E/E_0) in Mb  where
   *	F(y) = [(y-1)^2 + y_w^2] y^{-Q} (1 + sqrt{y/y_a})^{-P}
   *  HYDROGENIC: Hydrogenic
   *  CLARK: Clark,  Cowen, and Bobrowicz, ADNDT 1986, 34, 415-422
   */

int sigma_type; 
double IonE, kT;
double rrc_ph_factor, rrc_erg_factor;

int Zel, nq, lq;                                      /* For HYDROGENIC case */
double yw=0, Q=0, ya=0, P=0, E_0=0, E_th=0, sigma0=0; /* For VERNER case */
double frac=0,ip=0,n1=0,b1=0,d1=0,b2=0,d2=0,c[4];     /* For CLARK case */

void calc_rrc(int Z, int rmJ, double abund, double ion_pop, 
	      double I_e, double g_ratio, int sig_type, double *sigma_coeff, 
	      double T, double min_emission,
	      struct BIN_SPEC *spectrum, double *emission, double *rr_lev_pop){

  int iBin, l1;
  int neval, ierror;
  double E;
  double rrc_emission;
  double abserr;
  double rrc_check;
  double lbin, rbin;

  *rr_lev_pop = 0.;
  IonE = I_e;

  /* Set up units for RRC */
  rrc_ph_factor = (RRC_COEFF/ERG_KEV)*g_ratio/(T*sqrt(T)); 
  rrc_erg_factor= RRC_COEFF*abund*ion_pop*g_ratio/(T*sqrt(T)); 

  /* Set up sigma function call global variables */
  sigma_type = sig_type;
  switch (sig_type) {
  case (NO_PHOT):
    break;
  case (VERNER):
    E_th   = sigma_coeff[0];
    E_0    = sigma_coeff[1];
    sigma0 = sigma_coeff[2];
    ya     = sigma_coeff[3];
    P      = sigma_coeff[4];
    yw     = sigma_coeff[5];
    l1     = sigma_coeff[6];
    Q      = 5.5 + l1 - 0.5*P;
    break;
  case (HYDROGENIC):
    /* Hydrogenic approximation; only need quantum numbers n and l */
    nq = (int) sigma_coeff[0];
    lq = (int) sigma_coeff[1];
    Zel = Z;
    break;
  case (CLARK):
    /* ************************************************************* */
    /* Sept 7, 2005: Added following line, Zel = Z.  Apparently this */
    /* has been missing for a long time, which may make RR rates     */
    /* incorrect.  Need to check this properly!                      */
    /* (Not sure what Zel equals in this case.)                      */
    /* ************************************************************* */
    Zel = Z;
    frac=sigma_coeff[0]; /* Fraction, level_deg / sum(equiv. config deg) */
    ip = sigma_coeff[2];
    n1 = sigma_coeff[3];
    b1 = sigma_coeff[4];
    d1 = sigma_coeff[5];
    b2 = sigma_coeff[6];
    d2 = sigma_coeff[7];
    c[0] = sigma_coeff[8];
    c[1] = sigma_coeff[9];
    c[2] = sigma_coeff[10];
    c[3] = sigma_coeff[11];
    break;
  default:
    errmess("calc_rrc","Illegal sigma type = %d",sig_type);
    break;
  }

  kT = KBOLTZ*T;  /* in keV */

  /* Now we're set up.  So we first calculate the total recombination
     rate, and then (if spectrum != NULL), we calculate the RRC as well. */
  
  /*
   * Next is to add up the total level population induced by these 
   * recombinations.  This is done by doing an analytic integral 
   * over sigma (see 10/21/98 notes on RRC in documentation directory, 
   * or Shu Vol. I, eq 10.23).   Integrate from ionE (plus epsilon, in
   * this case 0.1 eV) to infinity 
   */

  *rr_lev_pop = dqagi(rrc_ph_value, I_e+1.e-4, 1, 
		      TOT_ABSACC, TOT_RELACC, &abserr, &neval, &ierror);
  /* in recominations cm^3 / s; multiply by n_I n_e to get total recomb. */

  /* Don't forget to check result flags */
  if ((ierror != 0)&&
      (ierror != 2)) { /* Ignore roundoff problems */
    message("calc_rrc","Error in indefinite integral, ierror=%d",ierror);
    message("calc_rrc",".... neval=%d, abserr=%e",neval, abserr);
    message("calc_rrc",".... %s, rrc_rate=%e",roman_element(Z,rmJ),
	    *rr_lev_pop);
  }

  /* Now to calculate the total emitted power by integrating over each bin. */
  rrc_check = 0.0;
  if ((spectrum != NULL)&&                          /* Results wanted */
      (*rr_lev_pop*ion_pop*abund > min_emission)) { /* and significant */
    lbin = -1.0; /* Left bin edge */
    for (iBin=0;iBin<spectrum->Nbin;iBin++) {
      if (spectrum->bin_edge[iBin+1] > I_e) { /* Past ionization edge */
	if (spectrum->bin_edge[iBin] > I_e) { /* WELL past ionization edge */
	  /* Before doing full numerical integral, check value at each bin */
	  /* bin edge; if change is less than BIN_RELACC, just use average.*/
	  if (lbin < 0) {
	    lbin = rrc_erg_value(spectrum->bin_edge[iBin]);
	  } else {
	    lbin = rbin; /* Reuse previous value. */
	  }
	  rbin =  rrc_erg_value(spectrum->bin_edge[iBin+1]);
	  if (fabs(2*(rbin - lbin)/(rbin + lbin)) < BIN_RELACC) {
	    rrc_emission = 0.5*(lbin + rbin)*
	      (spectrum->bin_edge[iBin+1] - spectrum->bin_edge[iBin]);
	  } else {  /* Do full calculation */
	    rrc_emission = 
	      dqags(rrc_erg_value, 
		    spectrum->bin_edge[iBin], spectrum->bin_edge[iBin+1],
		    BIN_ABSACC, BIN_RELACC, &abserr, &neval, &ierror);
	  }
	} else {  /* edge is in this bin; tricky */
	  lbin = -1; /* Reset lbin if it happened to be set */
	  rrc_emission = 0.0;
	  if ((I_e + 1e-4) < spectrum->bin_edge[iBin+1]) {
	    rrc_emission = 
	      dqags(rrc_erg_value, I_e+1e-4, spectrum->bin_edge[iBin+1],
		    BIN_ABSACC, BIN_RELACC, &abserr, &neval, &ierror);
	  }
	}	  
	if (ierror != 0) {
	  message("calc_rrc",
		  "%s recombinations, definite integral, ierror=%d, n=%d",
		  roman_element(Z,rmJ),ierror,neval);
	}
	E = (spectrum->bin_edge[iBin] + spectrum->bin_edge[iBin+1])/2.0;

	rrc_check += rrc_emission/(E*ERG_KEV*ion_pop*abund); 

	/* Units here are photons cm^3 / s */
	(spectrum->rrc_emiss[Z])[iBin] += rrc_emission/(E*ERG_KEV);
	(spectrum->bin_emission[Z])[iBin] += rrc_emission/(E*ERG_KEV);
	/* Units for emission is ergs cm^3 / s */
	spectrum->total_emission[Z] += rrc_emission;
	spectrum->tot_rrc_emis[Z] += rrc_emission;
	emission[iBin] += rrc_emission;
      }
    }
  }
  if (rrc_check > 0) {
    message("calc_rrc","intRR, sumRR, ratio: %s %e %e %f",
	    roman_element(Z, rmJ), *rr_lev_pop,rrc_check,
	    *rr_lev_pop/rrc_check);
  }
}


double sigma_photoion(double energy) {

  int ii;
  double result=0.0;
  double x, term1, term2, sum1, sum2;
  double F_y, y;

  switch (sigma_type) {
  case (NO_PHOT):
    result = 0.0;
    break;
  case (VERNER):
    if (energy > E_th) {
      y = energy/E_0;
      F_y = ((y-1)*(y-1) + yw*yw)*pow(y,-Q)*pow(1 + sqrt(y/ya),-P);
      result = sigma0*F_y*1.e-18;  /* in cm^2 */
    } else {
      result = 0.0;
    }
    break;
  case (HYDROGENIC):  /* Hydrogenic */
    result = sigma_hydrogenic(Zel, nq, lq, energy);
    break;
  case (CLARK):  /* Clark, Cowen, & Bobrowicz */
    if (energy > IonE) {
      x = energy/IonE;
      term1 = 1./((Zel + b1 + d1/Zel)*(Zel + b1 + d1/Zel));
      sum1 = 0.0;
      for (ii=0;ii<n1;ii++) sum1 += c[ii]*pow(x,ip+ii);
      term2 = 1./((Zel + b2 + d2/Zel)*(Zel + b2 + d2/Zel));
      sum2 = 0.0;
      for (ii=n1;ii<4;ii++) sum1 += c[ii]*pow(x,ip+ii);
      result = frac*(term1*sum1 + term2*sum2);
    } else {
      result = 0.0;
    }
    break;
  default:
    errmess("calc_rrc","Illegal sigma type %d",sigma_type);
    break;
  }
  return result;
}

double rrc_ph_value(double energy) {

  double result;
  /* Note only two factors of energy since results are in ph cm^3/s */
  result = rrc_ph_factor*sigma_photoion(energy)*energy*energy*
    exp(-(energy - IonE)/kT);
  return result;
  
}

double rrc_erg_value(double energy) {

  double result;
  result = rrc_erg_factor*sigma_photoion(energy)*energy*energy*energy*
    exp(-(energy - IonE)/kT);
  return result;
  
}
