#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "messages.h"
#include "apec.h"

/*! \file calc_two_phot.c */

void write_debug(struct DEBUG_GENERIC *debug, int ascii, int fits, 
		 char* fmt, ...);
void compress_energy_spectrum(int Nbin, double *bin_edge, double *emission, 
			      double error, char *type,
			      int *Nval, double **Eval, double **Fluxval);
double A_twoph(double A, double E0, double E);

/*! \brief Calculate two-photon emission and put it into spectrum output
 *  Required twophot structure is calculated by calc_emission().  If 
 *  spectrum == NULL, routine is skipped. 
 *  \param debug Debugging output files [optional; set to NULL to ignore]
 *  \param state Atomic data for recombined ion
 *  \param twophot Structure holding two photon transition rates 
 *  \param spectrum Spectral output structure [optional; set to NULL to ignore]
 */ 

void calc_two_phot(struct DEBUG_FILES *debug, 
		   struct ELEMENT_ION_STATE *state,
		   struct TWOPHOT_DATA *twophot, struct BIN_SPEC *spectrum) {

  double E0, A, lev_pop, E, dE, A_E;
  double *emission, tot_emiss_twophot;
  struct COMPRESS_CONT *cs_ptr;
  int Z, rmJ, iBin;

  if (spectrum == NULL) return; /* Nothing to calculate */

  if (twophot->exists) {
    tot_emiss_twophot = 0.;
    E0 = HC_IN_KEV_A/twophot->lambda;
    A  = twophot->A;
    lev_pop = twophot->lev_pop;
    Z   = twophot->Z;
    rmJ = twophot->rmJ;
    emission = (double *) malloc((spectrum->Nbin)*sizeof(double));

    for (iBin=0;iBin<spectrum->Nbin;iBin++) {
      E = 0.5*((spectrum->bin_edge)[iBin]+(spectrum->bin_edge)[iBin+1]);
      emission[iBin] = 0.0;
      if (E < E0) {
	dE = (spectrum->bin_edge)[iBin+1]-(spectrum->bin_edge)[iBin];
	A_E = A_twoph(A, E0, E);
	emission[iBin] = lev_pop*(E/E0)*A_E*dE*ERG_KEV; 
	                 /* this is in erg cm^3 / s / bin */
	(spectrum->twoph_emiss[Z])[iBin] += emission[iBin]/(E*ERG_KEV);
	(spectrum->bin_emission[Z])[iBin] += emission[iBin]/(E*ERG_KEV);
	spectrum->total_emission[Z] += emission[iBin];
	spectrum->tot_twoph_emis[Z] += emission[iBin];
	tot_emiss_twophot += emission[iBin];
      }
    }

    /* Now, compress the spectrum */
    cs_ptr = spectrum->cs;
    while ((cs_ptr != NULL) && ((cs_ptr->Z != Z) || (cs_ptr->rmJ != rmJ))) 
      cs_ptr = cs_ptr->next;
    if (cs_ptr == NULL) 
      errmess("calc_two_phot","Ion spectrum for %d, %d not found?",Z,rmJ);
    
    for (iBin=0;iBin<spectrum->Nbin;iBin++) 
      cs_ptr->bin_real[iBin] += emission[iBin];
    compress_energy_spectrum(spectrum->Nbin, spectrum->bin_edge, emission, 
			     ERROR_TWOPH, "Two Photon", &(cs_ptr->Ntwoph),
			     &(cs_ptr->E_twoph), &(cs_ptr->twoph));

    free(emission);

    if (debug != NULL) {
      if (debug->do_twoph) {
	write_debug(&debug->twoph, debug->ascii, debug->fits,
		    "%4d %4d %10g %10g\n",
		    Z, twophot->rmJ, A, tot_emiss_twophot);
	if (debug->ascii) fflush(debug->twoph.ascii);
      }
    }
  }
}

double A_twoph(double A, double E0, double E) {

  /* From Nussbaumer & Schmutz, 1984, A+A, 138,495 */
  /* Z is the element, and E is the energy of the bin, in keV */
  /* y is unitless, and is equal to nu/nu0 = lambda0/lambda, where */
  /* lambda0 = 1215.7 A for hydrogen--the base wavelength of the 2s->1s */
  /* transition.  This fit is accurate to better than 0.6% for */
  /* 0.01 < y < 0.99 */

  /* The A_norm is the A value for neutral hydrogen for this transition. */
  /* For other transitions, we renormalize to the appropriate A value. */

  /* This routine is used for BOTH hydrogenic and He-like two-photon */
  /* distributions.  This is justified using the result of */
  /* Derevianko & Johnson, 1997, Phys Rev A, 56, 1288 who show in */
  /* Figures 5 and 2 of that paper that the difference is everywhere */
  /* less than 10% between these two for Z=6-28 -- it is about 5% or so. */

  double C     = 202.0;  /* s^-1 */
  double alpha = 0.88;
  double beta  = 1.53;
  double gamma = 0.80;
  double A_norm= 8.2249; /* in s^-1 */
  double x, y, z;
  double result;
  
  y = E/E0;

  x = y*(1-y);
  z = pow(4*x,gamma);
  result = C*(x*(1-z) + alpha*pow(x,beta)*z);
  
  result *= (A/A_norm);  /* Also need R_Z/R_H, but even for Z=26, this is */
                         /* only 1.0005, so we'll ignore it. */
  return result;  /* in s^-1 */
}
