PRO write_ioniz, filename, Z, rmJ, data, COMMENTS=comments
;
;  Writes a IONIZ FITS file using the data in the structure data, for
;  element Z and ion stage rmJ.  The code first checks to see if
;  filename exists.  If it doesn't, it creates filename and then
;  writes the primary header.  If it does, the code checks to make
;  sure this particular ion stage hasn't already been written to the
;  file.  Assuming all is OK, we are positioned at the end of the file.
;  Then the header is created, and the data is written to the file.
;

;
;  Check to see if file exists
;
result = findfile(filename,count=count)
if (count eq 0) then begin      ; must create file and primary HDU
    fxhmake, primary_header, /extend, /initialize, /date
    fxaddpar, primary_header, 'COLL_ION','Collisional Ionization Rates'
    fxaddpar, primary_header, 'FILENAME','IDL_routine','Source of data'
    fxaddpar, primary_header, 'ORIGIN','CXC','Chandra X-ray Center'
    fxaddpar, primary_header, 'HDUCLASS','ATOMIC','Atomic data'
    fxaddpar, primary_header, 'HDUCLAS1','COLL_ION',$
      'e + p Collisional Ionization rates'
    version = '1.1.0'
    fxaddpar, primary_header, 'HDUVERS1', version, 'Version of datafile'
    fxwrite, filename, primary_header
endif

if (count gt 1) then begin
    print,'More than 1 file exists with this name: ',result
    stop
endif

if (count eq 1) then begin
    Ext = match_header(filename, Z, rmJ, 'COLL_ION')
    if (Ext gt 0) then begin
        print,'Extension ',Ext,' already has data for Z, rmJ = ',Z,rmJ,$
          ' in file ',filename
        return
    endif
endif
;
;  Now we're all set, ready to create the new header.
;
integer = 1L
float = 1.0
array = fltarr(20)
strings = '01234567890123456789'
nrows = n_elements(data)
extname = ionname(Z,rmJ)
fxbhmake, header, nrows, extname, 'Ion Name'
fxaddpar, header, 'HDUCLASS', 'ATOMIC', 'Atomic Data'
fxaddpar, header, 'HDUCLAS1','COLL_ION','Collisional Ionization rates'
fxaddpar, header, 'HDUCLAS2','ION_STATE','Individual Ion State'
fxaddpar, header, 'ELEMENT', Z, 'Number of protons in element'
fxaddpar, header, 'ION_STAT',rmJ-1,'Ion state (0 = neutral)'
fxaddpar, header, 'ION_NAME',extname, 'Ion Name'
fxaddpar, header, 'N_ION',nrows,'Number of collisional ionization rates'
version = '1.1.0'
fxaddpar, header, 'HDUVERS1', version, 'Version of datafile'
fxbaddcol, 2, header, integer, 'Ion_Final'
fxbaddcol, 3, header, integer, 'Level_Final'
fxbaddcol, 1, header, integer, 'Level_Init'
fxbaddcol, 4, header, integer, 'Tr_index'
fxbaddcol, 5, header, integer, 'Coll_Type'
fxbaddcol, 6, header, float,   'Min_Temp'
fxbaddcol, 7, header, float,   'Max_Temp'
fxbaddcol, 8, header, array,   'Temperature'
fxbaddcol, 9, header, array,   'Coll_Par'
fxbaddcol,10, header, strings, 'Sh_Label'
fxbaddcol,11, header, strings, 'Reference'
;
; Now add any additional comments
;
for i=0,n_elements(comments)-1 do begin
    if (strpos(comments(i),'COMMENT') eq 0) then begin
        commstr = strmid(comments(i),8)
        add_reference, header, commstr, /header
    endif else begin
        add_reference, header, comments(i), /header
    endelse
endfor
;
;  Now open the file and write the new header
;
fxbcreate, unit, filename, header
;
;  Write the data
;
structnames = tag_names(data)

for i=0L,nrows-1 do begin
    fxbwrite, unit, data(i).Ion_Final  , 1, i+1
    fxbwrite, unit, data(i).Level_Final, 2, i+1
    fxbwrite, unit, data(i).Level_Init , 3, i+1
    fxbwrite, unit, data(i).LineIndex  , 4, i+1
    fxbwrite, unit, data(i).Coll_Type  , 5, i+1
    fxbwrite, unit, data(i).Min_Temp   , 6, i+1
    fxbwrite, unit, data(i).Max_Temp   , 7, i+1
    fxbwrite, unit, data(i).Temperature, 8, i+1
    fxbwrite, unit, data(i).Coll_Par   , 9, i+1
    fxbwrite, unit, data(i).sh_label   ,10, i+1
    fxbwrite, unit, data(i).Reference  ,11, i+1
endfor
;
; Close the file
;
fxbfinish, unit

return
end
