%
% Routines to fix the temperature, density, normalization, and
% abundance of a certain element or a certain ion. 
%
%
% Ronnie Hoogerwerf, SAO, 05/10/2002
%

uses("SL-ll","SL-io");
% Function declarations.
%
define check_dataset_range();    % part of AM-model.sl
define AM_parse_dataset_range(); % part of AM-model.sl
define print_datasets ();
private define levels_array();

define AM_fix() {

   %
   % Add a element/ion to the list of fixed lines
   %

   variable usage = "AM_fix(Dataset,Temp,Dens,Norm,Abund,Z[,ion])";
   variable new_fix = @FixLine_Type;
   variable dataset,temp,dens,norm,abund,Z,ion = NULL;

   switch (_NARGS)
   { case 6 : (dataset,temp,dens,norm,abund,Z) = (); }
   { case 7 : (dataset,temp,dens,norm,abund,Z,ion) = (); }
   { message(usage); _pop_n(_NARGS); return;}
      
   if (Integer_Type == typeof(dataset)) {
      dataset = [dataset];
   } else if (String_Type == typeof(dataset)) {
      if (dataset == "all") {
	 dataset = [0];
      } else {
	 dataset = AM_parse_dataset_range(dataset);
      }
   } else {
      message("AM->fix: received incorrect dataset.");
   }
   if (typeof(Z) == String_Type) {
      if (assoc_key_exists(AM->Elem_to_Z,strlow(Z))) {
	 Z = AM->Elem_to_Z[strlow(Z)];
      } else {
	 verror("Element %s is not part of the apec model",Z);
      }
   }
   if (typeof(ion) == String_Type) {
      if (assoc_key_exists(AM->Roman_to_I,strlow(ion))) {
	 ion = AM->Roman_to_I[strlow(ion)];
      } else {
	 verror("Ion %s is not part of the apec model",ion);
      }
   }
   set_struct_fields(new_fix,dataset,temp,dens,norm,abund,Z,ion);

   AM->fix = LL->append(new_fix,AM->fix);



}

variable _TEMP_ = "1";
variable _DENS_ = "2";
variable _NORM_ = "3";
variable _ABUN_ = "4";

private define get_property(prop) {

   if (typeof(prop) == Integer_Type) prop = string(prop);
  
   switch (prop) 
   { case _TEMP_ : return "temperature"; }
   { case _DENS_ : return "density"; }
   { case _NORM_ : return "normalization"; }
   { case _ABUN_ : return "abundance"; }
   { return ""; }

}

define AM_change_fix() {

   variable nth, fix, nchange, values = Assoc_Type[], prop, val;
   variable usage = "AM_change_fix(line number, _PROP_, value, ...);\n  where _PROP_ is one of _TEMP_, _DENS_, _NORM_, or _ABUND_.";
   
   % Check the number of arguments.
   %
   if (_NARGS < 3 or Math->is_even(_NARGS)) {
      message(usage);
      _pop_n(_NARGS);
      return;
   } 

   % Get the number of properties that have to be changes and loop
   % over the stack to retrieve them. Perform some sanity checks
   % with appropriate error messages.
   %
   nchange = (_NARGS - 1) / 2;
   loop (nchange) {
      (prop, val) = ();
      if (typeof(prop) == Integer_Type) {
	 switch (prop)
	 { case 1 : prop = _TEMP_; }
	 { case 2 : prop = _DENS_; }
	 { case 3 : prop = _NORM_; }
	 { case 4 : prop = _ABUN_; }
	 { verror("Error: AM_change_fix(): Invalid value for prop (%d)",prop); }
      }
      if (NULL == Utils->member(prop,[_TEMP_,_DENS_,_NORM_,_ABUN_])) {
	 verror("Error: AM_change_fix(): properties (%s) is not one of _TEMP_, _DENS_, _NORM_, or _ABUND_.",string(prop));
      } else {
	 if (assoc_key_exists(values,prop)) {
	    vmessage("WARNING: AM_change_fix(): You are setting the %s property more than one time.\n         Assuming the first value as the appropriate value.",
		     get_property(prop));
	 } 
	 values[prop] = val;
      }
   }
   % get information of which line to change
   %
   nth = ();

   % Check if the requested line number exists. If not throw an error.
   % 
   if (nth > LL->length(AM->fix) - 1 or nth < 0) {
      verror("Error: AM_change_fix(): \n  You are requesting a change of line %d,\n  but only %d fixed lines are defined.",nth,LL->length(AM->fix));
   }
 
   % Get the requested line from the fixed line repository
   %
   fix = LL->nth(nth,AM->fix);

   % Make the appropriate change
   %
   foreach (values) using ("keys","values") {
      (prop,val) = ();
      switch(prop) 
      { case _TEMP_ : fix.temp = val; }
      { case _DENS_ : fix.dens = val; }
      { case _NORM_ : fix.norm = val; }
      { case _ABUN_ : fix.abund = val; }
   }

}

define AM_reset_fix () {

   %
   % Reset the fixed lines. This command discards of all
   % elements/lines that have been fixed.
   %

   AM->fix = NULL;

}

define AM_show_fix () {

   % 
   % Displays the information of the lines that are fixed and also 
   % provides the command to discard of a certain line.
   %

   variable tmpfix = AM->fix, fix, i = 0, remove;

   if (tmpfix == NULL) {
      message("There are no fixed lines");
      return;
   }

   message(" #  Z      ion  Temp      Dens      Norm      Abund            Datasets | To remove");
   message("==========================================================================================");
   foreach (tmpfix) {
      fix = ();

      remove = sprintf("AM_remove_fix(%d)",i);
      vmessage("%2d  %2s  %6s  %f  %f  %f  %f  %15s | %s", 
	       i,
	       AM_strZ(fix.content.Z),
	       AM_strIon(fix.content.ion),
	       fix.content.temp,
	       fix.content.dens,
	       fix.content.norm,
	       fix.content.abund,
	       print_datasets(fix.content.dataset),
	       remove);
      i++;
   }

}

define AM_remove_fix(i) {

   %
   % Remove the ith entry from the list of fixed elements/ions.
   %

   AM->fix = LL->remove(i,AM->fix);

}

define AM_get_free_lines(dataset) {

   %
   % Returns the line-list of all elements and ions that have
   % not been fixed.
   %

   if (AM->fix == NULL and AM->remove == NULL) {
      return NULL;
   }

   variable tmpfix = AM->fix, fix;
   variable lines, lines_str, i, n;

   lines_str = "where(";

   % handle fixed lines

   if (AM->fix != NULL) {
      i = 1;
      n = LL->length(AM->fix);
      foreach (tmpfix) {
	 
	 fix = ();
	 
	 if (Utils->member(dataset,fix.content.dataset) != NULL or
	     fix.content.dataset[0] == 0) {
	    
	    if (fix.content.ion == NULL) {
	       lines_str += sprintf("not(isis->el_ion(%d))",
				    fix.content.Z);
	    } else {
	       lines_str += sprintf("not(isis->el_ion(%d,%d))",
				    fix.content.Z,
				    fix.content.ion);
	    }
	    if (i < n) lines_str += " and ";
	    i++;
	 }
      } 
   }

   % handle removed lines
   if (AM->remove != NULL) {
      i = 1;
      n = LL->length(AM->remove);
     
      if (NULL != AM->fix) lines_str += "and ";

      variable tmpremove = AM->remove, remove;
      foreach (tmpremove) {

	 remove = ();
	 
	 if (Utils->member(dataset,remove.content.dataset) != NULL or
	     remove.content.dataset[0] == 0) {
	    
	    lines_str += sprintf("not(isis->trans(%d,", remove.content.Z);
	    if (remove.content.ion == NULL) {
	       lines_str += "NULL,";
	    } else {
	       lines_str += sprintf("%d,",remove.content.ion);
	    }
	    lines_str += sprintf("%s,%s)",
				 levels_array(remove.content.ul),
				 levels_array(remove.content.ll));
	    lines_str += ")";
	    
	    if (i < n) lines_str += " and ";
	    i++;
	 }
      }
   }

   lines_str += ");";
   if (lines_str == "where();") return NULL;
   lines = eval(lines_str);

   return lines;

}

define AM_get_fixed_line_flux(dataset,lo,hi) {

   %
   % Returns the flux of the spectral lines associated with the
   % set of fixed elements and ions.
   %

   if (AM->fix == NULL) {
      return NULL;
   }

   variable tmpfix = AM->fix, fix, model, flux, i ,n;
   variable lines, lines_str = "", lines_str_remove = "";

   % handle removed lines
   if (AM->remove != NULL) {
      i = 1;
      n = LL->length(AM->remove);
     
      variable tmpremove = AM->remove, remove;
      foreach (tmpremove) {

	 remove = ();
	 
	 if (Utils->member(dataset,remove.content.dataset) != NULL or
	     remove.content.dataset[0] == 0) {
	    
	    lines_str_remove += sprintf("not(isis->trans(%d,", 
					remove.content.Z);
	    if (remove.content.ion == NULL) {
	       lines_str_remove += "NULL,";
	    } else {
	       lines_str_remove += sprintf("%d,",remove.content.ion);
	    }
	    lines_str_remove += sprintf("%s,%s)",
				 levels_array(remove.content.ul),
				 levels_array(remove.content.ll));
	    lines_str_remove += ")";
	    
	    if (i < n) lines_str_remove += " and ";
	    i++;
	 }
      }
   }

   if (AM->fix != NULL) {

      foreach (tmpfix) {
	 
	 fix = ();
	 
	 if (Utils->member(dataset,fix.content.dataset) != NULL or
	     fix.content.dataset[0] == 0) {
	    
	    model = isis->default_plasma_state();
	    if (fix.content.temp  != NULL) model.temperature = 
		fix.content.temp/AM->K_to_keV;
	    if (fix.content.dens  != NULL) model.density = fix.content.dens;
	    if (fix.content.norm  != NULL) model.norm = fix.content.norm;
	    if (fix.content.abund != NULL) model.metal_abund = fix.content.abund;
	    
	    if (fix.content.ion == NULL) {
	       lines_str = sprintf("where(isis->el_ion(%d)",
				   fix.content.Z);
	    } else {
	       lines_str = sprintf("where(isis->el_ion(%d,%d)",
				   fix.content.Z,
				   fix.content.ion);
	    }
	    if (lines_str_remove != "") {
	       lines_str = lines_str + " and " + lines_str_remove + ");";
	    } else {
	       lines_str += ");";
	    }
	    lines = eval(lines_str);
	    
	    isis->define_model(model);
	    
	    if (__is_initialized(&flux) == 0) {
	       flux = isis->model_spectrum(lo,hi,isis->MODEL_LINES,lines);
	    } else {
	       flux += isis->model_spectrum(lo,hi,isis->MODEL_LINES,lines);
	    }
	 }
      } 
   }


   IO->write_block("flux.txt",lo,hi,flux);

   if (__is_initialized(&flux) == 0) {
      flux = Double_Type[length(lo)];
      flux[*] = 0.0;
   }
   
   return flux;

}


define print_datasets (arr) {

   variable ell;
   variable str = "";


   foreach (arr) {
      ell = ();
      if (ell == 0) {
	 ell = "all";
      } else {
	 ell = string(ell);
      }

      str = str + ell + ",";
   }

   return substr(str,1,strlen(str)-1);

}

private define levels_array(levels) {

   variable str = "[", l, i, n;

   if (levels == NULL) return "NULL";
   
   n = length(levels) - 1;
   for (i = 0; i <= n; i++) {

      str = str + string(levels[i]);
      if (i == n) {
	 str = str+"]";
      } else {
	 str = str+",";
      }
   }

   return str;

}

