%
% SLang implementation of some of the routines defined in the 
% Numerical Recipes (Second Edition).
%
% R. Hoogerwerf, SAO, 04/25/2002
%

uses("MATH-nrecip");
use_namespace("Math");


%-----

define erf(x) {

   % 
   % Returns value of the errorfunction:
   %
   % erf(x) = 2/\sqrt{\pi} * \int\limits_{0}^{x} exp^{-t^2} dt
   %


   if (x < 0) {
      return -gammp(0.5,x^2);
   } else {
      return gammp(0.5,x^2);
   }

}


%-----

define erfc(x) {

   %
   % Returns the complementary errorfunction of x (1 - erf(x))
   %
   % erfc(x) = 2/\sqrt{\pi} * \int\limits_{x}^{\infty} exp^{-t^2} dt
   %

   if (x < 0.0) {
      return 1.0 + gammp(0.5,x^2);
   } else {
      return gammq(0.5,x^2);
   }

}

%-----

define erfcc(x) {

   %
   % Returns the complementary errorfunction of x (1 - erf(x)) with
   % fractional error. Everywhere < 1.2x10-7
   %
   % erfc(x) = 2/\sqrt{\pi} * \int\limits_{x}^{\infty} exp^{-t^2} dt
   %
   % This code is adopted from Randall Smith. It basically is the 
   % numerical recipes code but allows the argument x to be an scalar 
   % as well as an array.
   %

   variable t, z, ans, neg, i;

   z = abs(x);
   t = 1.0/(1.0+0.5*z);

   ans = t*exp(-z*z-1.26551223+t*(1.00002368+t*(.37409196+t*(.09678418+
		  t*(-.18628806+t*(.27886807+t*(-1.13520398+t*(1.48851587+
                  t*(-.82215223+t*.17087277)))))))));

   if (typeof(x) == Array_Type) {
      neg = where(x < 0);
      if (length(neg) != 0) {
	 for (i=0;i<length(neg);i++) ans[neg[i]] = 2.0-ans[neg[i]];
      }
   } else {
      if (x < 0.0) ans = 2.0 - ans;
   }

   return ans;

}

%-----

define gammp(a,x) {

   if ((x < 0.) or (a <= 0.)) {
      error("Bad arguments in Math->gammp");
   }

   variable gammser, gammcf, gln;

   if (x < a+1.) {
      (gammser, gln) = gser(a,x);
      return gammser;
   } else {
      (gammcf, gln) = gcf(a,x);
      return 1.0 - gammcf;
   }
}

%-----

define gammq(a,x) {

   variable gammq, gammser, gammcf, gln;

   if ((x < 0.0) or (a <= 0.0)) error("bad arguments in gammq");
   if (x < a+1) {
      (gammser,gln) = gser(a,x);
      gammq = 1.0 - gammser;
   } else {
      (gammcf, gln) = gcf(a,x);
      gammq = gammcf;
   }

   return gammq;

}

%-----

define gser(a,x) {

   variable ITMAX = 100, EPS = 3.e-7;
   variable gammser, gln, n;

   variable ap, del, sum;

   gln = gammln(a);
   
   if (x <= 0.) {
      if (x < 0.) {
	 error("x < 0 in Math->gser");
      }
      gammser = 0.0;
      return gammser, gln;
   }

   ap = a;
   sum = 1./a;
   del = sum;
   
   for (n = 1; n <= ITMAX; n++) {
      
      ap  += 1.0;
      del *= x/ap;
      sum += del;

      if (abs(del) < abs(sum)*EPS) break;

   }

   if (n == ITMAX + 1) {
      error("a too large, ITMAX too small in gser");
   }

   gammser = sum * exp(-x+a*log(x)-gln);

   return gammser, gln;

}

%-----

define gcf(a,x) {

   variable ITMAX = 100, EPS = 3.0e-7, FPMIN = 1.0e-30;
   variable gammcf, gln;
   variable i, an, b, c, d, del, h;

   gln = gammln(a);
   b = x + 1.0 - a;
   c = 1.0 / FPMIN;
   d = 1.0 / b;
   h = d;
   for (i = 1; i <= ITMAX; i++) {

      an = -i*(i-a);
      b += 2.0;
      d = an*d + b;
      if (abs(d) < FPMIN) d = FPMIN;
      c = b + an/c;
      if (abs(c) < FPMIN) c = FPMIN;
      d = 1./d;
      del = d*c;
      h = h*del;
      if (abs(del - 1.0) < EPS) break;
   }

   if (i == ITMAX + 1) error("a too large, ITMAX too small in gcf");

   gammcf = exp(-x + a*log(x) - gln)*h;

   return gammcf, gln;

}

%-----

define gammln(xx) {

   variable j;
   variable ser, tmp, x, y;
   variable cof = [76.18009172947146e0,-86.50532032941677e0,
		   24.01409824083091e0,-1.231739572450155e0,
		   .1208650973866179e-2,-.5395239384953e-5];
   variable stp = 2.5066282746310005e0;

   x = xx;
   y = x;
   tmp = x + 5.5;
   tmp = (x+0.5)*log(tmp)-tmp;
   ser = 1.000000000190015e0;
   for (j = 0; j < 6; j++) {
      y += 1.0;
      ser += cof[j]/y;
   }
   return tmp+log(stp*ser/x);

}

%-----
%-----
%-----
%-----
%-----
%-----
