#define LIN_DENS 1
#define LOG_DENS 2
#define ARB_DENS 3
#define LIN_TEMP 1
#define LOG_TEMP 2
#define ARB_TEMP 3

#define HC_KEV 12.3984
#define MAXBIN 100000

#define LINES_ONLY 0
#define CONTINUUM_ONLY 1
#define LINES_AND_CONTINUUM 2


typedef struct line
{
  float kT;
  float density;
  long n;
  float *lambda;
  float *lambda_err;
  float *epsilon;
  float *epsilon_err;
  long *element;
  long *ion;
  long *upperlev;
  long *lowerlev;
  /* */
  long linelist_start;

} Line_Type;

typedef struct coco
{
  float kT;
  float density;
  long n;
  long *z;
  long *rmj;
  long *n_cont;
  long max_cont;
  float **e_cont;
  float **continuum;
  float **cont_err;
  long *n_pseudo;
  long max_pseudo;
  float **e_pseudo;
  float **pseudo;
  float **pseudo_err;
  
} Coco_Type;

typedef struct apecmodel 
{
  /* 
     File and naming specifics 
  */
  char *file_stem;        /* Stem of filenames loaded, e.g., apec_low_density_
                             This means apec_low_density_line.fits and
			     apec_low_density_coco.fits were read. */
  char *line_file;
  char *coco_file;
  char *model_name;       /* User-defined model name, defaults to file_stem */
  int model_index;        /* Unique running model number */
  /* 
     model specifics 
  */
  char ionbal[FLEN_KEYWORD];
  char filemap[FLEN_KEYWORD];
  char abundances[FLEN_KEYWORD];
  int nei;
  /* output options */
  char output_line[FLEN_KEYWORD], 
    output_coco[FLEN_KEYWORD], 
    output_comp[FLEN_KEYWORD], 
    output_bincont[FLEN_KEYWORD], 
    output_xspec[FLEN_KEYWORD];
  /* physical processes */
  int do_rrc, do_satel, do_lines, do_twoph, do_brems;
  char bremstype[FLEN_KEYWORD];
  /* binning */
  int bin_type;
  int num_bin;
  float bin_start, bin_end;
  float min_epsilon;
  /* physical parameters */
  int lin_dens;
  int num_dens;      /* Number of density components in model */
  float dens_start, dens_step;
  int lin_temp;
  int num_temp;           /* Number of kT components in model */
  float temp_start, temp_step;
  /* Used values and results. */
  int atoms[1000];
  long n;                  /* Number of components in model */
  float *kT;               /* Temperatures of model components */
  float Tmin, Tmax;
  float *density;          /* Densities of model components */
  float densitymin, densitymax;
  int **grid_kT_density;
  float *grid_kT;
  float *grid_density;
  long *Nelement;          /**/
  long Nline_total;        /**/
  long *Nline;             /**/
  long *Ncont;             /**/
  long *Npseudo;           /**/
  /* 
     line and coco for each model component 
  */
  Line_Type *line;
  Coco_Type *coco;
  /*
    Some useful extraneous stuff
  */
  float wvl_min, wvl_max;
  float kev_min, kev_max;
  float epsilon_min, epsilon_max;
  /* 
     pointer to next apec model in memory 
  */
  struct apecmodel *next;
} Apec_Model;

typedef struct emissivity_curve {
  long nT;
  long ndensity;
  float *kT;
  float *density;
  float **emissivity;
} Emissivity_Curve_Type;

/* Global variables */
static float solar_abundances[100] = {1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0};

/* function definitions */
int load_plasma(char *name, char *line_file, char *coco_file, char *file_stem, int verbose);
char *strtrim(char *string);
int decode_element(char *string);
int decode_ion(char *string);
int get_ionpop_indx(int Z,int rmnJ);
void free_line(Line_Type line);
int read_line_info(fitsfile *fp, Line_Type *line,long n);
void free_coco(Coco_Type coco);
int read_coco_info(fitsfile *fp, Coco_Type *coco,long n);
void free_am(Apec_Model *am);
Apec_Model *setup_kT_density_grid(Apec_Model *am);
Apec_Model *get_new_apec_model_ptr();
Apec_Model *get_apec_model_with_name(char *name);
Apec_Model *get_apec_model_with_index(int index);
void free_all_apec_models();
void show_apec_models();

int *find_grid_indices(Apec_Model *am, float T, float rho, int *is);

int extract_line_spectrum(Apec_Model *am,float kT,float density,float norm,float *abundances,float redshift,int keV,long nlinelist,long *linelist,int nbin,float *bin_lo,float *bin_hi,float *spectrum,int *indx);
int extract_coco_spectrum(Apec_Model *am,float kT,float density,float norm,float *abundances,int keV,int nbin,float *bin_lo,float *bin_hi,float *spectrum,int *indx);
int extract_spectrum(Apec_Model *am,float kT,float density,float norm,float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *bin_lo,float *bin_hi,float *spectrum);
int extract_ncomp_spectrum(Apec_Model *am,int ncomp,float *kT,float *density,float *norm,float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *bin_lo,float *bin_hi,float *spectrum);
int extract_spectrum_for_modelname(char *name,float kT,float density,float norm,float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *bin_lo,float *bin_hi,float *spectrum);
int extract_ncomp_spectrum_for_modelname(char *name,int ncomp,float *kT,float *density,float *norm,float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *bin_lo,float *bin_hi,float *spectrum);


long *_elem_ion_(char *model_name,long nelem,long *elem,long nion,long *ion,long *nout);
long *_wvl_(char *model_name,float min_wvl,float max_wvl,int keV,long *nout);
long *_emiss_(char *model_name,float min_emiss,float max_emiss,long *nout);
long *_ll_ul_(char *model_name,long nll,long *ll,long nul,long *ul,long *nout);
long *_make_linelist_(long n,long *which,long *nout);
long *get_linelist_for_component(Line_Type *line,long n,long *linelist,long *nend);


void free_emissivity_curve(Emissivity_Curve_Type *emiss_curve);
Emissivity_Curve_Type *emissivity_curve(Apec_Model *am,long element,long ion,long upperlev,long lowerlev);



/* */
int _em_power_law_(char *name, float gamma,float ampl,int nT,float Tmin,float Tmax,int log, float density, float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *binlo, float *binhi, float *spectrum);
/* */
int _em_broken_power_law_(char *name, float gamma1,float gamma2,float Tbreak,float ampl,int nT,float Tmin,float Tmax,int log, float density, float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *binlo, float *binhi, float *spectrum);
/* */
int _em_double_power_law_(char *name, float gamma,float beta, float alpha, float Tbreak,float ampl,int nT,float Tmin,float Tmax,int log, float density, float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *binlo, float *binhi, float *spectrum);
/* */
int _em_chebyshev_(char *name,int n,float *ampl,int nT,float Tmin,float Tmax,int log,float density,float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *binlo, float *binhi,float *spectrum);
/* */
int _em_polynomial_(char *name,int n,float *ampl,int nT,float Tmin,float Tmax,int log,float density,float *abundances,float redshift,int flag,int keV,long nlinelist,long *linelist,int nbin,float *binlo, float *binhi,float *spectrum);
/* */
