#include <stdlib.h>
#include <stdio.h>
#define __USE_GNU          /* this is to use strndup */
#include <string.h>
#include <math.h>
#include <limits.h>

#include "apec.h"
#include "_apec_slang.h"

/************************************************************************

   Routines to select specific lines. All routines return, in the OUT
   parameter an array of zeros and ones. The array will be of length
   Sum_components Nline_per_component. So it has a lot of double counting.
   However, it is the only way I can think of that makes the APEC output
   independent from the specific APED database that was used to calculate
   it. This independence will alow users to make individual changes to the
   APED/APEC model and still use the linelist capability.

   These are NOT the list of indices that are input to the extract_spectrum
   routines. They are mainly there to make use of the WHERE facility in 
   S-Lang. For the C-code we will provide a routine to make the linelist
   (see below).

**************************************************************************/


long *_elem_ion_(char *model_name, 
		 long nelem, 
		 long *elem, 
		 long nion, 
		 long *ion, 
		 long *nout) {
  

  long *out;
  Apec_Model *am;
  Line_Type *line;
  long i, j, k, l, nline, tmp;

  *nout = 0; 
  if (NULL == (am = get_apec_model_with_name(model_name))) {
    printf("Could not find APEC model with name : %s\n",model_name);
    return(NULL);
  }

  /* get total number of lines in am */
  nline = 0;
  for (i=0;i<am->n;i++) nline += am->Nline[i];
  /* printf("Total number of lines in APEC model is %ld\n",nline); */
  
  *nout = nline;
  /* allocate memory and initialize, strating with all lines included */
  out = (long *)malloc(sizeof(long)*nline);
  for (i=0;i<nline;i++) out[i] = 1;

  /* Now select the correct lines */
  /* First loop over the elements */
  if (nelem > 0) {
    i=0;
    for (k=0;k<am->n;k++) {             /* Loop over model components */
      line = &(am->line[k]);
      for (l=0;l<am->Nline[k];l++) {    /* Loop over lines in component */
	tmp = 0;
	for (j=0;j<nelem;j++) {	      /* Loop over elements to be included */
	  if (line->element[l] == elem[j]) tmp = 1;
	}
	if (!tmp) out[i] = 0; 
	i++;
      }
    }
  }

  /* First loop over the ions */
  if (nion > 0) {
    i=0;
    for (k=0;k<am->n;k++) {             /* Loop over model components */
      line = &(am->line[k]);
      for (l=0;l<am->Nline[k];l++) {    /* Loop over lines in component */
	tmp = 0;
	for (j=0;j<nion;j++) {	      /* Loop over elements to be included */
	  if (out[i] == 1 && line->ion[l] == ion[j]) tmp = 1;
	}
	if (!tmp) out[i] = 0; 
	i++;
      }
    }
  }

  /* just a check */
  /* l = 0;
     for (i=0;i<nline;i++) if (out[i] == 1) l++;
     printf("Found %ld lines that satisfy the constraint\n",l); 
  */

  return(out);

}


long *_wvl_(char *model_name, 
	    float min_wvl,
	    float max_wvl,
	    int keV,
	    long *nout) {
  

  long *out;
  Apec_Model *am;
  Line_Type *line;
  long i, k, l, nline;
  float min_wvl_, max_wvl_;

  *nout = 0; 
  if (NULL == (am = get_apec_model_with_name(model_name))) {
    printf("Could not find APEC model with name : %s\n",model_name);
    return(NULL);
  }

  if (min_wvl == -1.0) min_wvl = am->wvl_min;
  if (max_wvl == -1.0) max_wvl = am->wvl_max;

  min_wvl_ = keV ? HC_KEV/max_wvl : min_wvl;
  max_wvl_ = keV ? HC_KEV/min_wvl : max_wvl;

  /* get total number of lines in am */
  nline = 0;
  for (i=0;i<am->n;i++) nline += am->Nline[i];
  /* printf("Total number of lines in APEC model is %ld\n",nline); */
  
  *nout = nline;
  /* allocate memory and initialize, strating with all lines included */
  out = (long *)malloc(sizeof(long)*nline);
  for (i=0;i<nline;i++) out[i] = 1;

  /* Now select the correct lines */
  i=0;
  for (k=0;k<am->n;k++) {             /* Loop over model components */
    line = &(am->line[k]);
    for (l=0;l<am->Nline[k];l++) {    /* Loop over lines in component */
      if (line->lambda[l] < min_wvl_ ||
	  line->lambda[l] > max_wvl_) out[i] = 0;
      i++;
    }
  }

  /* just a check */
  /* l = 0;
     for (i=0;i<nline;i++) if (out[i] == 1) l++;
     printf("Found %ld lines that satisfy the constraint\n",l); */


  return(out);

}


long *_emiss_(char *model_name, 
	      float min_emiss,
	      float max_emiss,
	      long *nout) {

  long *out;
  Apec_Model *am;
  Line_Type *line;
  long i, k, l, nline;

  *nout = 0; 
  if (NULL == (am = get_apec_model_with_name(model_name))) {
    printf("Could not find APEC model with name : %s\n",model_name);
    return(NULL);
  }

  if (min_emiss == -1.0) min_emiss = am->epsilon_min;
  if (max_emiss == -1.0) max_emiss = am->epsilon_max;

  /* get total number of lines in am */
  nline = 0;
  for (i=0;i<am->n;i++) nline += am->Nline[i];
  /* printf("Total number of lines in APEC model is %ld\n",nline); */
  
  *nout = nline;
  /* allocate memory and initialize, strating with all lines included */
  out = (long *)malloc(sizeof(long)*nline);
  for (i=0;i<nline;i++) out[i] = 1;

  /* Now select the correct lines */
  i=0;
  for (k=0;k<am->n;k++) {             /* Loop over model components */
    line = &(am->line[k]);
    for (l=0;l<am->Nline[k];l++) {    /* Loop over lines in component */
      if (line->epsilon[l] < min_emiss || 
	  line->epsilon[l] > max_emiss) out[i] = 0;
      i++;
    }
  }

  /* just a check */
  /* l = 0;
     for (i=0;i<nline;i++) if (out[i] == 1) l++;
     printf("Found %ld lines that satisfy the constraint\n",l); */


  return(out);

}


long *_ll_ul_(char *model_name, 
	      long nll, 
	      long *ll, 
	      long nul, 
	      long *ul, 
	      long *nout) {
  
  long *out;
  Apec_Model *am;
  Line_Type *line;
  long i, j, k, l, nline, tmp;

  *nout = 0; 
  if (NULL == (am = get_apec_model_with_name(model_name))) {
    printf("Could not find APEC model with name : %s\n",model_name);
    return(NULL);
  }

  /* get total number of lines in am */
  nline = 0;
  for (i=0;i<am->n;i++) nline += am->Nline[i];
  /* printf("Total number of lines in APEC model is %ld\n",nline); */
  
  *nout = nline;
  /* allocate memory and initialize, strating with all lines included */
  out = (long *)malloc(sizeof(long)*nline);
  for (i=0;i<nline;i++) out[i] = 1;

  /* Now select the correct lines */
  /* First loop over the elements */
  if (nll > 0) {
    i=0;
    for (k=0;k<am->n;k++) {             /* Loop over model components */
      line = &(am->line[k]);
      for (l=0;l<am->Nline[k];l++) {    /* Loop over lines in component */
	tmp = 0;
	for (j=0;j<nll;j++) {	      /* Loop over elements to be included */
	  if (line->lowerlev[l] == ll[j]) tmp = 1;
	}
	if (!tmp) out[i] = 0;
	i++;
      }
    }
  }

  /* First loop over the ions */
  if (nul > 0) {
    i=0;
    for (k=0;k<am->n;k++) {             /* Loop over model components */
      line = &(am->line[k]);
      for (l=0;l<am->Nline[k];l++) {    /* Loop over lines in component */
	tmp = 0;
	for (j=0;j<nul;j++) {	      /* Loop over elements to be included */
	  if (out[i] == 1 && line->upperlev[l] == ul[j]) tmp = 1;
	}
	if (!tmp) out[i] = 0;
	i++;
      }
    }
  }


  /* just a check */
  /* l = 0;
     for (i=0;i<nline;i++) if (out[i] == 1) l++;
     printf("Found %ld lines that satisfy the constraint\n",l); */
  
  return(out);

}


long *_make_linelist_(long n, long *which, long *nout) {

  long *out;
  long i, m;

  m = 0;
  for (i=0;i<n;i++) {
    if (which[i] == 1) m++;
  }
  *nout = m;
  out = (long *)malloc(sizeof(long)*m);

  m = 0;
  for (i=0;i<n;i++) {
    if (which[i] == 1) {
      out[m] = i;
      m++;
    }
  }

  return(out);

}

long *get_linelist_for_component(Line_Type *line, 
				 long n, 
				 long *linelist, 
				 long *nend) {

  long first_element = line->linelist_start;
  long last_element = first_element + line->n - 1;
  long *start, m;
  long i;

  i = 0;
  while (linelist[i] < first_element && i<n) i++;
  start = &(linelist[i]);
  
  m = 0;
  while (linelist[i+m] < last_element && i+m<n) m++;
  *nend = m;

  return(start);

}
