C Author: Randall Smith
C Date: 11/11/93
C Version: 7.0
C Language: Fortran 77
C Routine: sputter
C Invocation: sputter(abund,T,rho,chi,deltat,F,X,dustms,dustcool)
C Purpose: Calculates the sputtering of dust and the cooling from same, via
c collisions with protons and heavier ions.  Cooling approximation comes from
c stopping power of electrons in sand.
C
C  input parameters:
C	T	temperature (units: K)
C	rho	mass density (units: amu/cm**3)
C	deltat	timestep (units: sec); subdivided as necessary
C	X(*)	ionstate vector; indexed by index(no)
C	F(*)	Fraction sputtered off of type 1 or 2 grains.
C       P(*)      exponent of dust size distribution, core type i
C       Rmin(*)   Minimum radius of type i dust
C       Rmax(*)   Maximum radius of type i dust
C       Frac(a,b) Fractional elemental abundance of element b, core a
C                 (a=1: Type I, a=2: Type 2, a=3: Mantle)
C       dustrho density of average dust particle, in gm/cm**-3
C       Porosity  of silicate dust
C       dene     electron density in cm**-3
C       dend    number density of dust, in cm**-3
C       dustms(*) total mass of dust in each grain, in grams/ H atom
C       AlogT = common log of temperature
C       mu = mean mass per nucleus (atomic units)  !not used
C       abhe = abundance of He by number           !not used
C       H0..Heplus = # of such atoms per electron  !not used
C       chi = # of free particles per nucleus
C       delta_a = distance sputtered off radii
C       a_min = lower limit of dust radii, changing as dust sputters
C       a_max = upper limit of dust radii, changing as dust sputters
C       a_weight = atomic weight of element number i
C       A = normalization constant such that with the size distribution
C           dN/da = Aa**(-P) initially, one satisfies cosmic abundances.
C
C Revisions: Many.  Original version used 
C Side Effects: COMMON blocks are used.  Beware.
C Other Notes: None.
C End:

      SUBROUTINE sputter(abund,T,rho,chi,deltat,F,X,dustms,dustcool)

      IMPLICIT NONE

      include 'evolve.h'
c
c     sputter7.f version is like sputter2 except that it uses 
c                Romberg integration to do the H integral faster. 

      real P,Rmin,Rmax,Frac,dustrho,Porosity
      common/dust/P(2),Rmin(2),Rmax(2),Frac(3,num),dustrho(3),Porosity
      real dustms(2), dustcl(2)
      common/dustparms/dustcl
      real F(2),T,rho,deltat,X(Nions),Y(Nions),dustcool
      real Sc(2),CRatio,Vc1,Vc2
      real alogT,dene,chi,ab,newabund,abund(num)
      real delta_a(2), a_min(2), a_max(2), A(2), r0(2)
      real a_crit,N_A
      real a_weight(num), dummy, Rof,secyr,kB
      real Tcgra,Tcsil,beta
      integer i,j
      parameter(Tcgra = 3.8E6, Tcsil = 6E6)
      parameter(N_A=6.022E23)  ! particles/mole 
      parameter(secyr=3.15E7)  ! sec/yr
      parameter(kB=8.614E-5)   ! in ev/K
      real SiSput,CSput,cool_int
      external SiSput,CSput,cool_int
      data a_weight/1.00,4.00,12.0,14.0,16.0,20.2,24.3,28.1,32.1,
     #              39.9,40.1,55.8,58.7,27.0/
C      data a_weight/1.00,4.00,12.011,14.007,15.999,20.17,24.305,
C     $     28.09,32.06,39.948,40.08,55.847,58.71,26.98/
C Calculate A once when no dust has yet sputtered, i.e. F=0.
c Note that:
C *    dustms(j) = Fraction in dust * #atom i/ H atom * atomic
C      weight of atom i in grams
c *    0.2387 = ((4/3)*pi)**-1 
c *    Before the conversion, the units are
c         [A] = cm**3 * Angstoms**(P(j)-4) / H atom.

      if(F(1).EQ.0) then
         do j = 1, 2
            dustms(j) = 0 
            do i = 1, num
               dustms(j) = dustms(j) + Frac(j,i) * a_weight(i) / N_A
            end do
            A(j) = dustms(j) * (.2387*(4-P(j))/dustrho(j))
            if (j.eq.1) A(j) = A(j)/(1.-Porosity)
            A(j) = A(j) / (Rmax(j)**(4-P(j)) - Rmin(j)**(4-P(j)))            
            A(j) = A(j) * 1.0e24 !Convert to Angstoms**(P(j)-1) / H atom
c            write(*,*) 'Dust mass = ',dustms(j)
c            write(*,*) 'Rmax, Rmin = ',Rmax(j), Rmin(j)
c            write(*,*) 'Dust norm ',j,'=',A(j)
         end do
c         write(*,*) 'Dustmass, Silicate, g/Hatom) : ',dustms(1)
c         write(*,*) 'Dustmass, Graphite, g/Hatom) : ',dustms(2)
c         write(*,*) 'A, silicate, (A^p-1 grains/H atom) : ',A(1)
c         write(*,*) 'A, graphite, (A^p-1 grains/H atom) : ',A(2)
      endif
c
c     Set up some other parameters.
c     Note that dene = electron density; rho = amu/cc (mass density)
c
      dene = (chi-1.0)*(1.1/1.4)*rho
      alogT = log10(T)

C     a_crit = largest grain radius for which a significant
C     number of electrons pass through, thus lowering
C     the heat donated to the grain.
C     Rof = tells which H function to use in integral.  
C     1. = normal function when h=1
C     2. = multiply by (a/a_crit)**.5 since some e- pass through. 


C  output parameters:
C	dustms(2)  	new mass of dust, in grams 
C	dustcool  	cooling rate (1.e-23 erg cm**3 /sec) 
C	X(*)		ion concentration rate
C	abund		abundance of elements normalize to 10^12 H.

C     see pg 402, interstellar processes, for dust composition info.

C
C     First, calculate how much dust is sputtered in this parcel, given
C     the temperature, density, and electron density.  The dust should be
C     then broken down by element, with the assumed elemental composition
C     possibly depending on the 1) size of grains 2) type of grains 3)
C     amount of dust left. 
C
C     Type I dust : Silicates
C     Type II dust: Carbon grains, used in this version 
C     Type III is a mantle on the silicates (not currently used)
C
C     Sputtering is assumed to be the dominant effect (ie, this is not a
C     radiative shock, so grain-grain, betatron are not important) and the
C     data is taken from Seab, Interstellar Processes, p. 503.  A first fit
C     was done to the graph there, covering the Draine data for both Si and
C     graphite.  The data was taken off the graph and interpolated, and 
C     the interpolations are stored in SiSput() and CSput() functions.


      Vc1=(1./(1.-Porosity))*(dene/(chi-1))*10.**(SiSput(AlogT))
      Vc2 = (dene/(chi-1))*10.**(CSput(AlogT))

C Now convert these to Angstroms/sec

      Vc1 = Vc1/secyr
      Vc2 = Vc2/secyr

C Set up the new independent variables.

c      delta0=(MantleThickness)/Rmax(1)
      F(1) = F(1) + (Vc1/Rmax(1))*deltat
      if(F(1).GT.1.0) F(1) = 1.0
c      Fm = (Vc1/Vm)*delta0
      F(2)= F(2) + (Vc2/Rmax(2))*deltat 
      if(F(2).GT.1.0) F(2) = 1.0

      delta_a(1) = (F(1)*Rmax(1)) 
      delta_a(2) = (F(2)*Rmax(2))

      dustcool = 0
      do 10 j = 1, 2
         a_min(j) = Rmin(j) - delta_a(j)
         if(a_min(j).LT.0) a_min(j) = 0
         a_max(j) = Rmax(j) - delta_a(j)
         if(a_max(j).LT.0) a_max(j) = 0
         
         
C     Calculate dustmass, the total mass in the dust at this time.
C     See Don's Feb 11, 1992 algebra notes for these formulae.
         
         dustms(j) = 4.*3.14159/3.* dustrho(j)*A(j)*1.e-24
         if (j.eq.1) dustms(j) = dustms(j)*(1.-Porosity)
         
         if(delta_a(j).LT.Rmin(j)) then
            dustms(j)=dustms(j)*(
     #           (Rmax(j)**(4-P(j))-Rmin(j)**(4-P(j)))/(4-P(j))  
     #           -3*delta_a(j)*(Rmax(j)**(3-P(j))
     $                         -Rmin(j)**(3-P(j)))/(3-P(j))
     #           +3*delta_a(j)**2*(Rmax(j)**(2-P(j))
     $                            -Rmin(j)**(2-P(j)))/(2-P(j))
     #           -1*delta_a(j)**3*(Rmax(j)**(1-P(j))
     $                            -Rmin(j)**(1-P(j)))/(1-P(j)))
         endif
         
         if(delta_a(j).GE.Rmin(j)) then
            dustms(j) = dustms(j) * (
     #           (Rmax(j)**(4-P(j))-delta_a(j)**(4-P(j)))/(4-P(j))
     #-3*delta_a(j)*(Rmax(j)**(3-P(j))-delta_a(j)**(3-P(j)))/(3-P(j))
     #+3*delta_a(j)**2*(Rmax(j)**(2-P(j))-delta_a(j)**(2-P(j)))/(2-P(j))
     #-1*delta_a(j)**3*(Rmax(j)**(1-P(j))-delta_a(j)**(1-P(j)))/(1-P(j))
     #    )
         endif
 
 
C  Dustcool = [1e-23 ergs*cm**3 per second ]
C  Do the integral, then multiply by the constants outside of it.
c  L_{dust} \propto \int_{min(a1-\delta,0)}^{a2-\delta}
c     a^2 (a+\delta)^{-3.5} (a/(a+\beta)) da.  
c  a^2 comes from the surface area, (a+\delta)^(-3.5) from the MRN dist.
c  of dust particles, and a/(a+\beta) is a supression term fit from figure 3
c  in Dwek, Apj 322, 812.  This is also the source of the 0.875 term, which
c  is unitless.
c
c  Note that 4 \sqrt{2 \pi} k_B^{3/2} / \sqrt{m_e} = 5.38 \times 10^{-10}
c  erg cm s^{-1} K^{-1.5}.  The integral has units of A^{-0.5}, and A(j)
c  has units of \AA^{2.5}.  
c  So, we need to convert from \AA^{2} to \cm^{2} by multiplying by
c  10^{-16}.  So, we have 5.38 \times 10^{-26} erg cm^3 s^{-1} K^{-1.5}.
c  But we want the answer in units of 10^{-23} erg cm^3 s^{-1}, so we 
c  multiply by 10^{23} and get 5.38 \times 10^{-3} erg cm^3 s^{-1} K^{-1.5}

         dustcl(j) = 0

         if (j.eq.1) then
            beta = (1.-Porosity)*100*(T/Tcsil)**1.75
         else
            beta = 100*(T/Tcgra)**1.75
         end if
         
         dustcl(j) = cool_int(a_min(j),a_max(j),beta,delta_a(j))
         dustcl(j) = dustcl(j) * 0.875*((5.38e-03) * T**1.5 * A(j))
         dustcool = dustcool + dustcl(j)

 10   continue


C Now set r0.
      if (Rmin(1).GT.F(1)*Rmax(1)) r0(1) = Rmin(1)
      if (Rmin(1).LE.F(1)*Rmax(1)) r0(1) = F(1)*Rmax(1)
      if (Rmin(2).GT.F(2)*Rmax(2)) r0(2) = Rmin(2)
      if (Rmin(2).LE.F(2)*Rmax(2)) r0(2) = F(2)*Rmax(2)

C Now calculate sputtering yields for each core type.
      DO 20 i=1,2
         CRatio = r0(i)/Rmax(i)
         if(CRatio.lt.1) then
            Sc(i) = (1/(4-P(i)))*(1-CRatio**(4-P(i)))
         Sc(i) = Sc(i) - (3*F(i)/(3-P(i)))*(1-CRatio**(3-P(i)))
         Sc(i) = Sc(i) + (3*(F(i)**2)/(2-P(i)))*(1-CRatio**(2-P(i)))
         Sc(i) = Sc(i) - ((F(i)**3)/(1-P(i)))*(1-CRatio**(1-P(i)))
         Sc(i) = (4-P(i))*Sc(i)/(1-CRatio**(4-P(i)))
         else
            Sc(i) = 0
         endif
 20   CONTINUE


C     Now, renormalize X and recalculate the densities of the various
C     elements.  Start by calculating Y, a vector that is the abundance of
C     each stage, relative to 10**12 for HI.

      DO 40 j=1,num
         IF (index(j).NE.0) then
            ab = 10.**(abund(j)-12.)
            DO 30 i=0,nj(j)
               Y(index(j)+i) = ab*X(index(j)+i)
 30         CONTINUE
            newabund = abundinit(j) + (1-Sc(1))*Frac(1,j)
            newabund = newabund     + (1-Sc(2))*Frac(2,j)
c           newabund = newabund     +    (1-Sm)*Frac(3,j)
            Y(index(j))= Y(index(j)) + (newabund-ab)
            abund(j) = newabund
         END IF
 40   CONTINUE

C     Renormalize X

      DO 60 j=1,num
         IF (index(j).NE.0) then
C
C Necessary to keep the X ionvector normalized correctly.
C
            newabund=0.0
            DO 45 i=0,nj(j)
               newabund=newabund+Y(index(j)+i)
 45         CONTINUE
            DO 50 i=0,nj(j)
               X(index(j)+i) = Y(index(j)+i)/newabund
c...........fix logarithmic abundance problem 4/28/92 rje
 50         CONTINUE
         END IF
         abund(j) = log10(abund(j)) + 12.
 60   CONTINUE

      RETURN
      END
c

c
C======================================================================

      REAL FUNCTION SISput(x)
c
c     2/25/93 RKS x = log10(T), returns the sputtering rate of carbon 
c                 particles for that temperature in Angstoms/yr.  
c                 From Seab's paper on the subject in Interstellar
c                 Processes, p. 503.
c     WARNING: Only good between 10^5 and 10^8K...answers given outside that
c              range are HIGHLY provisional, and above 10^8.1 it dies.
      implicit none
      integer points,i,nfit
      parameter(points=13,nfit=6)
      real logT(points),Sput(points),Slope,Result,x,xp
      real fit(nfit)

      data fit/-2.7446,1.5439,-0.37046,0.21641,-0.34755,0.10114/
      

      xp = x-6
      Result = ((((fit(6)*xp + fit(5))*xp + fit(4))*xp + fit(3))*xp
     $     + fit(2))*xp + fit(1)
      SiSput = Result

      return
      end



      REAL FUNCTION CSput(x)

c
c     2/25/93 RKS x = log10(T), returns the sputtering rate of carbon 
c                 particles for that temperature in Angstoms/yr.  
c                 From Seab's paper on the subject in Interstellar
c                 Processes, p. 503.  logT and Sput found by picking points
c                 of graph in that paper.
c     WARNING: Only good between 10^5 and 10^8K...answers given outside that
c              range are HIGHLY provisional, and above 10^8.1 it dies.

      implicit none
      integer points,i,nfit
      parameter(points=16,nfit = 6)
      real logT(points),Sput(points),Slope,Result,x,xp
      real fit(nfit)

      data fit/-2.8605,1.0572,-0.27545,0.23735,-0.31820,0.087376/

      xp = x-6
      Result = ((((fit(6)*xp + fit(5))*xp + fit(4))*xp + fit(3))*xp
     $     + fit(2))*xp + fit(1)
      CSput = Result

      return
      end



