#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>

#include "slang.h"

#define TRUE 1
#define FALSE 0
#define NUM 12
#define MAXBINS 10000
#define ALLION 220
#define MAXLINES 2000

static unsigned char _initialized;

int init_slrs93_module(void);
void setup_raymond_(void);

void slrs93_equil();
void rs93_equil_(float *T, float *dene);
void slrs93_nonequil();
void rs93_nonequil_(float *T, float *dene, float *dtime);
void slrs93_setbin();
void setup_bin_(int *Nbin, float *binmin, float *binsyz);
void slrs93_getresult();
void rs93_getdata_(float *, float *, float *, float *, 
		   float *, float *, float *, int *, 
		   float *, float *, float *, int *);
void slrs93_getlines();
void rs93_getlines_(float *rs93_wavelen, float *rs93_fosc, 
		    float *rs93_linepwr, int *rs93_atom, 
		    int *rs93_ion, int *rs93_index, int *rs93_numlines);

static SLang_Intrin_Fun_Type SLrs93_Intrinsics [] = 
  {
  MAKE_INTRINSIC_0("slrs93_equil", slrs93_equil, SLANG_VOID_TYPE),
  MAKE_INTRINSIC_0("slrs93_nonequil", slrs93_nonequil, SLANG_VOID_TYPE),
  MAKE_INTRINSIC_0("slrs93_setbin",slrs93_setbin, SLANG_VOID_TYPE),
  MAKE_INTRINSIC_0("slrs93_getresult",slrs93_getresult, SLANG_VOID_TYPE),
  MAKE_INTRINSIC_0("slrs93_getlines",slrs93_getlines, SLANG_VOID_TYPE),
  SLANG_END_INTRIN_FUN_TABLE
  };

int init_slrs93_module(void) {

  if (_initialized) return 0;
  _initialized = 1;

  /* Initialize RS93 */
  setup_raymond_(); 

  if (-1 == SLadd_intrin_fun_table (SLrs93_Intrinsics, NULL)) {
    return -1;
  }

  return 0;
}

void slrs93_equil() {
  
  double density, temperature;
  float T, dene;

  if (SLang_Num_Function_Args != 2) {
    SLang_verror(SL_USAGE_ERROR,
		 "Usage: slrs93_equil(T, dene)");
    return;
  }

  if( (SLang_pop_double(&density, NULL, NULL) == -1 ) ||
      (SLang_pop_double(&temperature, NULL, NULL) == -1 )) {
    return;
  }
  
  T = (float) temperature;
  dene = (float) density;
  
  rs93_equil_(&T, &dene);
  
}

void slrs93_nonequil() {
  
  double density, temperature, deltatime;
  float T, dene, dtime;

  if (SLang_Num_Function_Args != 3) {
    SLang_verror(SL_USAGE_ERROR,
		 "Usage: slrs93_nonequil(T, dene, dtime)");
    return;
  }

  if( (SLang_pop_double(&deltatime, NULL, NULL) == -1 ) ||
      (SLang_pop_double(&density, NULL, NULL) == -1 ) ||
      (SLang_pop_double(&temperature, NULL, NULL) == -1 )) {
    return;
  }
  
  T = (float) temperature;
  dene = (float) density;
  dtime = (float) deltatime;

  rs93_nonequil_(&T, &dene, &dtime);
  
}

void slrs93_setbin() {

  int Nbin;
  double binmin_dbl, binsyz_dbl;
  float binmin, binsyz;

  if (SLang_Num_Function_Args != 3) {
    SLang_verror(SL_USAGE_ERROR,
		 "Usage: slrs93_setbin(Nbin,binmin,binsyz)");
    return;
  }

  if( (SLang_pop_double(&binsyz_dbl, NULL, NULL) == -1 ) ||
      (SLang_pop_double(&binmin_dbl, NULL, NULL) == -1 ) ||
      (SLang_pop_integer(&Nbin) == -1 ) ) {
    return;
  }
  
  binmin = (float) binmin_dbl;
  binsyz = (float) binsyz_dbl;
  if (Nbin > MAXBINS) {
    SLang_verror(SL_USAGE_ERROR,
		 "Number of bins (%d) > maximum (%d)",Nbin, MAXBINS);
    return;
  }
    
  setup_bin_(&Nbin, &binmin, &binsyz);
  
}

void slrs93_getresult() {

  float rs93_pcool, rs93_pc[NUM], rs93_cnc[ALLION];
  float rs93_power[ALLION];
  float rs93_binmin, rs93_binsyz;
  float rs93_bin[MAXBINS], rs93_brmev[MAXBINS];
  float rs93_recev[MAXBINS], rs93_tufev[MAXBINS];
  int rs93_Nbin, rs93_nj[NUM];

  /* Outputs */
  SLang_Array_Type *sl_ionJ   = NULL;
  SLang_Array_Type *sl_cooling= NULL;
  SLang_Array_Type *sl_ionpop = NULL;
  SLang_Array_Type *sl_ionpow = NULL;
  SLang_Array_Type *sl_spec   = NULL;
  SLang_Array_Type *sl_brmev  = NULL;
  SLang_Array_Type *sl_recev  = NULL;
  SLang_Array_Type *sl_tufev  = NULL;

  /* Create structure to hold output results */
  char *field_names[12];
  unsigned char field_types[12];
  VOID_STAR field_values[12];
  int dims;
  int i;
  
  if (SLang_Num_Function_Args != 0) {
    SLang_verror (SL_USAGE_ERROR, "Usage: res = slrs93_getdata()");
    return;
  }

  rs93_getdata_(rs93_bin, rs93_brmev, rs93_recev, rs93_tufev, rs93_pc,
		rs93_cnc, rs93_power, rs93_nj, 
		&rs93_pcool, &rs93_binmin, &rs93_binsyz, &rs93_Nbin);

  /* transfer output values to s-lang array */
  field_names[0] = "totcool";field_types[0] = SLANG_FLOAT_TYPE;
  field_names[1] = "ionJ";   field_types[1] = SLANG_ARRAY_TYPE;
  field_names[2] = "cooling";field_types[2] = SLANG_ARRAY_TYPE;
  field_names[3] = "ionpop"; field_types[3] = SLANG_ARRAY_TYPE;
  field_names[4] = "ionpow"; field_types[4] = SLANG_ARRAY_TYPE;
  field_names[5] = "Nbin";   field_types[5] = SLANG_INT_TYPE;
  field_names[6] = "binmin"; field_types[6] = SLANG_FLOAT_TYPE;
  field_names[7] = "binsyz"; field_types[7] = SLANG_FLOAT_TYPE;
  field_names[8] = "spec";   field_types[8] = SLANG_ARRAY_TYPE;
  field_names[9] = "brmev";  field_types[9] = SLANG_ARRAY_TYPE;
  field_names[10]= "recev";  field_types[10]= SLANG_ARRAY_TYPE;
  field_names[11]= "tufev";  field_types[11]= SLANG_ARRAY_TYPE;
  
  field_values[0] = &rs93_pcool;
  field_values[5] = &rs93_Nbin;
  field_values[6] = &rs93_binmin;
  field_values[7] = &rs93_binsyz;

  /* These are real arrays */
  dims = NUM;
  /* sl_ionJ = SLang_create_array(SLANG_INT_TYPE, 0, rs93_nj, &dims, 1);*/
  sl_ionJ = SLang_create_array(SLANG_INT_TYPE, 0, NULL, &dims, 1);
  if ( sl_ionJ == NULL ) { return ;}
  /* sl_cooling = SLang_create_array(SLANG_FLOAT_TYPE, 0, rs93_pc, &dims, 1);*/
  sl_cooling = SLang_create_array(SLANG_FLOAT_TYPE, 0, NULL, &dims, 1);
  if ( sl_cooling == NULL ) { return ;}

  for (i=0;i<NUM;i++) {
    SLang_set_array_element( sl_ionJ, &i, &(rs93_nj[i]) );
    SLang_set_array_element( sl_cooling, &i, &(rs93_pc[i]) );
  }

  dims = ALLION;
  /*sl_ionpop = SLang_create_array(SLANG_FLOAT_TYPE, 0, rs93_cnc, &dims, 1);*/
  sl_ionpop = SLang_create_array(SLANG_FLOAT_TYPE, 0, NULL, &dims, 1);
  if ( sl_ionpop == NULL ) { return ;}
  /*sl_ionpow = SLang_create_array(SLANG_FLOAT_TYPE, 0,rs93_power, &dims, 1);*/
  sl_ionpow = SLang_create_array(SLANG_FLOAT_TYPE, 0,NULL, &dims, 1);
  if ( sl_ionpow == NULL ) { return ;}
  for (i=0;i<ALLION;i++) {
    SLang_set_array_element( sl_ionpop, &i, &(rs93_cnc[i]) );
    SLang_set_array_element( sl_ionpow, &i, &(rs93_power[i]) );
  }
  
  dims = rs93_Nbin;
  sl_spec = SLang_create_array(SLANG_FLOAT_TYPE, 0, rs93_bin, &dims, 1);
  sl_brmev= SLang_create_array(SLANG_FLOAT_TYPE, 0, rs93_brmev, &dims, 1);
  sl_recev= SLang_create_array(SLANG_FLOAT_TYPE, 0, rs93_recev, &dims, 1);
  sl_tufev= SLang_create_array(SLANG_FLOAT_TYPE, 0, rs93_tufev, &dims, 1);
  if ( sl_spec == NULL ) { return ;}


  field_values[1] = &sl_ionJ;
  field_values[2] = &sl_cooling;
  field_values[3] = &sl_ionpop;
  field_values[4] = &sl_ionpow;
  field_values[8] = &sl_spec;
  field_values[9] = &sl_brmev;
  field_values[10]= &sl_recev;
  field_values[11]= &sl_tufev;
  
  if (SLstruct_create_struct(12, field_names, field_types, field_values)==-1){
    return;
  }

  return;
}


void slrs93_getlines() {


  float rs93_wavelen[MAXLINES], rs93_fosc[MAXLINES];
  float rs93_linepwr[MAXLINES];
  int rs93_atom[MAXLINES], rs93_ion[MAXLINES], rs93_index[MAXLINES];

  /* Outputs */
  int rs93_numlines;
  SLang_Array_Type *sl_wavelen   = NULL;
  SLang_Array_Type *sl_fosc= NULL;
  SLang_Array_Type *sl_linepwr = NULL;
  SLang_Array_Type *sl_atom = NULL;
  SLang_Array_Type *sl_ion   = NULL;
  SLang_Array_Type *sl_index = NULL;

  /* Create structure to hold output results */
  char *field_names[7];
  unsigned char field_types[7];
  VOID_STAR field_values[7];
  int dims;
  int i;
  
  if (SLang_Num_Function_Args != 0) {
    SLang_verror (SL_USAGE_ERROR, "Usage: res = slrs93_getlines()");
    return;
  }

  rs93_getlines_(rs93_wavelen, rs93_fosc, rs93_linepwr,
		 rs93_atom, rs93_ion, rs93_index, &rs93_numlines);

  /* transfer output values to s-lang array */
  field_names[0] = "Nlines" ; field_types[0] = SLANG_INT_TYPE;
  field_names[1] = "atom"   ; field_types[1] = SLANG_ARRAY_TYPE;
  field_names[2] = "ion"    ; field_types[2] = SLANG_ARRAY_TYPE;
  field_names[3] = "index"  ; field_types[3] = SLANG_ARRAY_TYPE;
  field_names[4] = "wavelen"; field_types[4] = SLANG_ARRAY_TYPE;
  field_names[5] = "power"  ; field_types[5] = SLANG_ARRAY_TYPE;
  field_names[6] = "f_osc"  ; field_types[6] = SLANG_ARRAY_TYPE;
  
  field_values[0] = &rs93_numlines;

  /* These are real arrays */
  dims = rs93_numlines;

  sl_atom = SLang_create_array(SLANG_INT_TYPE, 0, NULL, &dims, 1);
  if ( sl_atom == NULL ) { return ;}
  sl_ion  = SLang_create_array(SLANG_INT_TYPE, 0, NULL, &dims, 1);
  if ( sl_ion == NULL ) { return ;}
  sl_index  = SLang_create_array(SLANG_INT_TYPE, 0, NULL, &dims, 1);
  if ( sl_index == NULL ) { return ;}

  for (i=0;i<rs93_numlines;i++) {
    SLang_set_array_element( sl_atom, &i, &(rs93_atom[i]) );
    SLang_set_array_element( sl_ion, &i, &(rs93_ion[i]) );
    SLang_set_array_element( sl_index, &i, &(rs93_index[i]) );
  }

  sl_wavelen = SLang_create_array(SLANG_FLOAT_TYPE, 0, NULL, &dims, 1);
  if ( sl_wavelen == NULL ) { return ;}
  sl_linepwr = SLang_create_array(SLANG_FLOAT_TYPE, 0,NULL, &dims, 1);
  if ( sl_linepwr == NULL ) { return ;}
  sl_fosc = SLang_create_array(SLANG_FLOAT_TYPE, 0,NULL, &dims, 1);
  if ( sl_fosc == NULL ) { return ;}

  for (i=0;i<rs93_numlines;i++) {
    SLang_set_array_element( sl_wavelen, &i, &(rs93_wavelen[i]) );
    SLang_set_array_element( sl_linepwr, &i, &(rs93_linepwr[i]) );
    SLang_set_array_element( sl_fosc,    &i, &(rs93_fosc[i]) );
  }
  
  field_values[1] = &sl_atom;
  field_values[2] = &sl_ion;
  field_values[3] = &sl_index;
  field_values[4] = &sl_wavelen;
  field_values[5] = &sl_linepwr;
  field_values[6] = &sl_fosc;
  
  if (SLstruct_create_struct(7, field_names, field_types, field_values)==-1){
    return;
  }

  return;
}

