/** \file AtomicLevel.cxx
    \brief Implementation of AtomicLevel class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <cstring>
#include <iomanip>
#include <sstream>

#include "profit/AtomicLevel.h"

extern "C" {
#include "apec.h"
}

namespace {
  // Local-only variable, used for default-constructed AtomicLevel to prevent bad access errors.
  LEVEL_DATA undefined_level;
}

namespace profit {

  AtomicLevel::AtomicLevel(): m_level_data(&undefined_level) {
    // One-time initialization.
    static bool first_time = true;
    if (first_time) {
      first_time = false;
      // Initialize undefined level to have no information.
      memset(&undefined_level, '\0', sizeof(LEVEL_DATA) / sizeof(char));
    }
  }

  AtomicLevel::AtomicLevel(const LEVEL_DATA & level_data): m_level_data(&level_data) {}

  double AtomicLevel::energy() const { return m_level_data->energy; }

  int AtomicLevel::quantumN() const { return m_level_data->N; }

  int AtomicLevel::quantumL() const { return m_level_data->L; }

  double AtomicLevel::quantumS() const { return m_level_data->S; }

  int AtomicLevel::degeneracy() const { return m_level_data->deg; }

  int AtomicLevel::level() const { return m_level_data->lev; }

  std::string AtomicLevel::config() const { return m_level_data->config; }

  std::string AtomicLevel::description() const {
    std::ostringstream os;
    os.precision(6);
    os << std::left << std::setw(9) << m_level_data->lev << std::setw(26) << config() << std::setw(7) << std::right << energy() <<
      std::setw(3) << quantumN() << std::setw(3) << quantumL() <<
      std::setw(4) << std::setprecision(3) << quantumS() << std::setw(3) << degeneracy();
    return os.str();
  }

}
