/** \file Os.cxx
    \brief Implementation of Os class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/Os.h"

#ifdef WIN32
#include <windows.h>
#endif

#include <cctype>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <stdexcept>

namespace {

#ifdef WIN32
  const std::string s_path_sep = ";";
#else
  const std::string s_path_sep = ":";
#endif
}

namespace profit {

  std::string Os::getEnv(const std::string & var_name) {
    const char * value = std::getenv(var_name.c_str());
    if (0 == value) throw std::runtime_error("Os::getEnv could not get environment variable \"" + var_name + "\"");
    return value;
  }

  void Os::setEnv(const std::string & var_name, const std::string & value) {
#ifdef WIN32
    int status = _putenv((var_name + "=" + value).c_str());
#else
    // TODO Fix memory leak here.
    char * env_string = new char[var_name.size() + 1 + value.size() + 1];
    std::strcpy(env_string, var_name.c_str());
    std::strcat(env_string, "=");
    std::strcat(env_string, value.c_str());

    int status = putenv(env_string);
#endif
    if (0 != status)
      throw std::runtime_error("Os::setEnv could not set environment variable \"" + var_name + "\" = \"" + value + "\"");
  }

  std::string Os::appendPath(const std::string & path1, const std::string & path2) {
#ifdef WIN32
    return path1 + ";" + path2;
#else
    return path1 + ":" + path2;
#endif
  }

  std::string Os::joinPathSegment(const std::string & segment1, const std::string & segment2) {
#ifdef WIN32
    return segment1 + "\\" + segment2;
#else
    return segment1 + "/" + segment2;
#endif
  }

  void Os::readEnv(const std::string & config_file_name) {
    std::ifstream fs(config_file_name.c_str());
    if (fs.good()) {
      static const unsigned int buf_size = 4096u;
      char buf[buf_size] = "";
      while (fs.good() && !fs.eof()) {
        // Blank buffer.
        *buf = '\0';

        // Get next line from file.
        fs.getline(buf, buf_size);

        // TODO How do we know the line isn't too long?

        // Skip leading space.
        char * cp = buf;
        while (0 != std::isspace(*cp)) ++cp;

        // Ignore blank lines and comments flagged with #.
        if ('\0' == *cp || '#' == *cp) continue;

        // Look for first equals sign.
        char * equals = std::strchr(cp, '=');

        // If no equals or equals is first, don't know how to parse this.
        if (0 == equals || cp == equals)
          throw std::runtime_error("Cannot parse line in profit configuration file " + config_file_name + ": " + buf);

        // Name is everything before equals except for leading and trailing space.
        char * name_begin = cp;
        for (cp = equals; cp > name_begin && 0 != std::isspace(*(cp - 1)); --cp);
        char * name_end = cp;

        // Skip space after equals.
        cp = equals + 1;
        while (0 != std::isspace(*cp)) ++cp;

        // Value is everything after equals except for leading and trailing space.
        char * value_begin = cp;
        for (cp = value_begin + std::strlen(value_begin); cp > value_begin && 0 != std::isspace(*(cp - 1)); --cp);
        char * value_end = cp;

        // Expand environment variable names.
        std::string value = expandEnv(std::string(value_begin, value_end));

        // Assign variable name = value.
        setEnv(std::string(name_begin, name_end), value);
      }
    } else {
      throw std::runtime_error("Cannot access profit configuration file: " + config_file_name);
    }
  }

  std::string Os::expandEnv(const std::string & text) {
    static const std::string s_var_begin_delim("${");
    static const std::string s_var_end_delim("}");
    static const unsigned int s_begin_delim_len(s_var_begin_delim.size());
    static const unsigned int s_end_delim_len(s_var_end_delim.size());

    std::string expanded;
    std::string::size_type begin = 0;
    // While there is text
    while (std::string::npos != begin) {
      // Find next delimiter indicating possible beginning of environment variable name.
      std::string::size_type var_begin = text.find(s_var_begin_delim, begin);
      // If found...
      if (std::string::npos != var_begin) {
        // Find delimiter indicating end of environment variable name.
        std::string::size_type var_end = text.find(s_var_end_delim, var_begin + s_begin_delim_len);
        // If found...
        if (std::string::npos != var_end) {
          // Copy all text before variable to expanded output.
          expanded += text.substr(begin, var_begin - begin);
          // Expand environment variable and add its text to expanded output.
          expanded += getEnv(text.substr(var_begin + s_begin_delim_len, var_end - var_begin - s_begin_delim_len));
          // Update position to continue string parsing after environment variable name.
          begin = var_end + s_end_delim_len;
        } else {
          // Copy all text before variable and including delimiter to expanded output.
          expanded += text.substr(begin, var_begin + s_begin_delim_len - begin);
          // Update position to continue string parsing after the environment variable delimiter.
          begin = var_begin + s_begin_delim_len;
        }
        // Make sure we haven't fallen off the end of the string.
        if (begin >= text.size()) begin = std::string::npos;
      } else {
        // Copy all remaining text to expanded output.
        expanded += text.substr(begin);
        // Update final position.
        begin = std::string::npos;
      }
    }
    return expanded;
  }
}
