/** \file PlasmaDb.cxx
    \brief Implementation of PlasmaDb class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/

#include <csetjmp>
#include <csignal>
#include <cstring>
#include <sstream>
#include <stdexcept>

#include "profit/PlasmaDb.h"

namespace {
  std::jmp_buf s_env;
}

extern "C" {

#include "apec.h"

#include "messages.h"
#include "apec_funcdefs.h"

void expand_env(char *filename);
void get_input_data(int argc, char *argv[], Param_File_Type *p_input,
                    struct PARAMETERS *params,
                    struct DEBUG_FILES *debug);

void create_spectral_bins(int BinType, int Nbin,
			  double BinMin, double BinMaX,
			  int N_Elements, int list_Z[],
			  struct BIN_SPEC *spectrum);
void get_atomic_data(char *filemap_file, int require_checksum,
		     int N_Elements, int *list_Z,
		     int *list_NrmJ, int *list_rmJ[],
		     int *maxZ, int *maxNlev,
		     ElementP **Atomic_params);
void get_abund(char *abund_source, char *filemap_file, double **abund);
void fits_open(struct PARAMETERS *params, ElementP *Atomic_data,
	       struct BIN_SPEC *spectrum);
void open_debug_output(struct DEBUG_FILES *debug);
void read_ion_bal(char *ionbal_file, struct DEBUG_FILES *debug,
		  struct ION_BAL_TABLE *ionbal);
void init_rate_matrices(int maxNlev);
void init_trans_rates(int maxsize, struct TRANS_RATES *trans_rates);

void initialize(struct PARAMETERS *params, struct BIN_SPEC *spectrum,
		ElementP **Atomic_data, double **abund,
		struct ION_BAL_TABLE *ionbal,
		struct TRANS_RATES *trans_rates,
		double **Lev_pop,
		struct DEBUG_FILES *debug, int *maxZ, int *maxNlev);

static void apec_do_not_abort(void) {
  std::longjmp(s_env, 1);
}

static void apec_do_not_exit(int) {
  std::longjmp(s_env, 1);
}

}

namespace profit {

  class PlasmaDb::Apec {
    public:
      Apec();
      ElementP * m_atomic_params;
      DEBUG_FILES m_debug;
      PARAMETERS m_parameters;
      BIN_SPEC m_spectrum;
      ION_BAL_TABLE m_ion_bal;
  };

  PlasmaDb::Apec::Apec() {
    memset(this, '\0', sizeof(Apec)/sizeof(char));
  }

  PlasmaDb & PlasmaDb::instance() {
    static PlasmaDb s_instance;
    return s_instance;
  }

  PlasmaDb::~PlasmaDb() { free(m_abund); delete m_apec; }

  AtomicLevel PlasmaDb::findLevel(int Z, int rmJ, int level) {
    static LEVEL_DATA s_not_found = { level, "not available", 0, -1, -1, -1., -1. };
    struct LEVEL_DATA * ptr = 0;

    // Test arguments.
    if (1 > Z || 1 > rmJ || 1 > level) {
      std::ostringstream os;
      os << "PlasmaDb::findLevel called for invalid level designation Z = " << Z << ", rmJ = " << rmJ << ", level = " << level;
      throw std::runtime_error(os.str());
    }

    // Set level of static struct used for dielectronic recombinations to agree with calling level.
    s_not_found.lev = level;

    // Levels 10000 and up are doubly excited, used for dielectronic recombination.
    // Return a well behaved struct with no info in this case. This is not an error, so this does not
    // throw an exception.
    if (level >= 10000) return AtomicLevel(s_not_found);

    for (int el_index = 0; el_index < m_apec->m_parameters.N_Elements; ++el_index) { /* Find the right data */
      if (Z == m_apec->m_parameters.list_Z[el_index]) {
        struct ELEMENT_ION_STATE * state = m_apec->m_atomic_params[el_index];
        int num_level = state[rmJ - 1].ion_inf->nlev;
        if (level > num_level) {
          std::ostringstream os;
          os << "PlasmaDb::findLevel cannot find atomic level Z = " << Z << ", rmJ = " << rmJ << ", level = " << level;
          throw std::runtime_error(os.str());
        }

        ptr = &(state[rmJ - 1].lev_dat[level - 1]);
        break;
      }
    }

    if (0 == ptr) {
      std::ostringstream os;
      os << "PlasmaDb::findLevel cannot find atomic level Z = " << Z << ", rmJ = " << rmJ << ", level = " << level;
      throw std::runtime_error(os.str());
    }

    return AtomicLevel(*ptr);
  }

  AtomicTransition PlasmaDb::findTransition(int Z, int rmJ, int lower_level, int upper_level) {
    // Test arguments.
    if (1 > Z || 1 > rmJ || 1 > lower_level || 1 > upper_level || lower_level >= upper_level) {
      std::ostringstream os;
      os << "PlasmaDb::findLevel called for invalid transition designation Z = " << Z << ", rmJ = " << rmJ <<
        ", lower_level = " << lower_level << ", upper_level = " << upper_level;
      throw std::runtime_error(os.str());
    }

    struct ELEMENT_ION_STATE * state = 0;

    AtomicLevel lower;
    AtomicTransition transition;

    if (0 == setjmp(s_env)) {

      if (upper_level >= 10000) {
        if (rmJ == 1)
          throw std::runtime_error("PlasmaDb::findTransition cannot find lower level for a neutral dielectronic transition");
        // Dielectronic transition means the lower level is actually for the next lower ion.
        lower = findLevel(Z, rmJ - 1, lower_level);
      } else {
        lower = findLevel(Z, rmJ, lower_level);
      }

      AtomicLevel upper = findLevel(Z, rmJ, upper_level);

      transition = AtomicTransition(Z, rmJ, lower, upper);

      for (int el_index = 0; el_index < m_apec->m_parameters.N_Elements; ++el_index) { /* Find the right data */
        if (Z == m_apec->m_parameters.list_Z[el_index]) {
          state = m_apec->m_atomic_params[el_index];
          int nexc = state[rmJ - 1].ion_inf->nexc;

          // Find collision info.
          for (int ii = 0; ii < nexc; ++ii) {
            struct COLLTRNS_DATA * ptr = &(state[rmJ - 1].colltrns_dat[ii]);
            if ((ptr->lev_lower == lower_level) && (ptr->lev_upper == upper_level)) {
              transition.addProcess(ptr);
            }
          }

          // Find proton rates.
          int npro = state[rmJ - 1].ion_inf->npro;
          for (int ii = 0; ii < npro; ++ii) {
            struct COLLTRNS_DATA * ptr = &(state[rmJ - 1].protrns_dat[ii]);
            if ((ptr->lev_lower == lower_level) && (ptr->lev_upper == upper_level)) {
              transition.addProcess(ptr);
            }
          }

          // Find radiative transitions.
          int nwv = state[rmJ - 1].ion_inf->nwv;
          for (int ii = 0; ii < nwv; ++ii) {
            struct LINERAD_DATA * ptr = &(state[rmJ - 1].linerad_dat[ii]);
            if ((ptr->lev_lower == lower_level) && (ptr->lev_upper == upper_level)) {
              transition.addProcess(ptr);
            }
          }

          // Find satellite transitions.
          int nsatel = state[rmJ - 1].ion_inf->nsatel;
          for (int ii = 0; ii < nsatel; ++ii) {
            struct SATELLITE_DATA * ptr = &(state[rmJ - 1].satellite_dat[ii]);
            if ((ptr->lev_lower == lower_level) && (ptr->lev_upper == upper_level)) {
              transition.addProcess(ptr);
            }
          }
          break;
        }
      }
    } else {
      throw std::runtime_error("PlasmaDb::findTransition: some problem with APEC; please report \n"
        "circumstances leading to this problem.");
    }
    return transition;
  }

  PlasmaDb::PlasmaDb(): m_apec(new Apec), m_abund(0) {
    // Reset Apec's message abort and exit so that it simply flags any errors.
    apec_set_abort(&apec_do_not_abort);
    apec_set_exit(&apec_do_not_exit);

    if (0 == setjmp(s_env)) {
      /* Read apec.par to get initial parameters. If it doesn't exist, create it */
      Param_File_Type * pf = 0;

      /* And now get the input data. */
      get_input_data(0, 0, pf, &m_apec->m_parameters, &m_apec->m_debug);

      /* Set up the truly never-changing stuff.  Need to use the filemap,
         which means we need to expand any variables first */
      expand_env(m_apec->m_parameters.filemap_file);

      /*  Similar to initialize.c */
      /*  Loads APED files and sets up parameters for a run, using the
          most current set of input parameters OR the passed-in values.
          Note that this should keep track of what has been read so that
          we don't unnecessarily re-read files.  */

      /* Create spectral bins (and fill in some parameters) */
      //create_spectral_bins(&m_apec->m_parameters, &m_apec->m_spectrum);
      PARAMETERS * params = &m_apec->m_parameters;
      int maxZ = 0;
      int maxNlev = 0;

      get_atomic_data(params->filemap_file, params->require_checksum,
  		  params->N_Elements, params->list_Z,
  		  params->list_NrmJ, params->list_rmJ,
  		  &maxZ, &maxNlev, &m_apec->m_atomic_params);

      /* Read atomic abundances */
      get_abund(params->abund_source, params->filemap_file, &m_abund);

      if (m_apec->m_parameters.ionbal_file[0] != '\0') {
        expand_env(m_apec->m_parameters.ionbal_file);
        read_ion_bal(m_apec->m_parameters.ionbal_file, &m_apec->m_debug, &m_apec->m_ion_bal);
      }
    } else {
      throw std::runtime_error("PlasmaDb::PlasmaDb: some problem with APEC; please report \n"
        "circumstances leading to this problem.");
    }
  }

} // namespace profit.
