/** \file RootFrame.cxx
    \brief Implementation of RootFrame class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/

#include "RootFrame.h"

#include "TGFrame.h"

#include <iostream>
#include <stdexcept>

namespace profit {

  RootFrame::RootFrame(TGFrame * frame, TGCompositeFrame * parent, ULong_t hints, Int_t pad_left,
    Int_t pad_right, Int_t pad_top, Int_t pad_bottom): m_frame(frame), m_parent(parent),
    m_layout_hints(new TGLayoutHints(hints, pad_left, pad_right, pad_top, pad_bottom)) {
    if (0 == m_frame) throw std::logic_error("RootFrame::RootFrame(TGFrame *, TGCompositeFrame *, ...) was passed a NULL pointer");
    if (0 != m_parent) {
      m_parent->AddFrame(m_frame, m_layout_hints);
    }
  }

  RootFrame::RootFrame(const RootFrame & frame): m_frame(0), m_parent(0), m_layout_hints(0) {
    assign(const_cast<RootFrame &>(frame));
  }

  RootFrame::~RootFrame() {
    if (0 != m_parent) {
      m_parent->RemoveFrame(m_frame);
    }
    delete m_layout_hints;
    delete m_frame;
  }

  TGLayoutHints * RootFrame::getHints() { return m_layout_hints; }

  RootFrame & RootFrame::operator =(const RootFrame & frame) {
    assign(const_cast<RootFrame &>(frame));
    return *this;
  }

  bool RootFrame::operator ==(const TGFrame * frame) const { return m_frame == frame; }

  void RootFrame::assign(RootFrame & frame) {
    // Transfer all members from the other frame.
    m_layout_hints = frame.m_layout_hints;
    m_frame = frame.m_frame;
    m_parent = frame.m_parent;

    // Clear other frame.
    frame.m_parent = 0;
    frame.m_frame = 0;
    frame.m_layout_hints = 0;
  }

  RootFrameTracker::~RootFrameTracker() { clear(); }

  void RootFrameTracker::trackFrame(const std::string & context, TGFrame * frame, TGCompositeFrame * parent, ULong_t hints,
    Int_t pad_left, Int_t pad_right, Int_t pad_top, Int_t pad_bottom) {
    if (0 == frame) throw std::runtime_error("Allocation of " + context + " failed");

    // Windows compiler has trouble getting the right "find" overload, so the following doesn't work:
    // std::list<RootFrame>::iterator itor = std::find(m_frame_cont.begin(), m_frame_cont.end(), frame);
    // Look for this frame in the container and track it if it is not already tracked.
    std::list<RootFrame>::iterator itor = m_frame_cont.begin();
    for (; itor != m_frame_cont.end() && !(*itor == frame); ++itor) {}
    if (itor == m_frame_cont.end())
      m_frame_cont.push_back(RootFrame(frame, parent, hints, pad_left, pad_right, pad_top, pad_bottom));
  }

  void RootFrameTracker::clear() {
    // Delete frames in reverse order of their creation.
    while (!m_frame_cont.empty()) m_frame_cont.pop_back();
  }

  RootFrame & RootFrameTracker::front() { return m_frame_cont.front(); }

  RootFrame & RootFrameTracker::back() { return m_frame_cont.back(); }

}
