/** \file StripChart.cxx
    \brief Implementation of StripChart class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <sstream>
#include <stdexcept>
#include <vector>

#include "TCanvas.h"
#include "TGButton.h"
#include "TGClient.h"
#include "TGFrame.h"
#include "TGLayout.h"
#include "TGMenu.h"
#include "TRootEmbeddedCanvas.h"

#include "TH1.h"

#include "profit/Gui.h"
#include "profit/SpectrumView.h"
#include "profit/StripChart.h"

#include <iostream>

ClassImp(profit::StripChart)

namespace profit {

  StripChart::StripChart(const SpectrumView * view, const TGWindow * main, UInt_t w, UInt_t h, unsigned int number_plots):
    m_view_cont(), m_hints_expand(kLHintsExpandX | kLHintsExpandY), m_main_frame(0),
    m_print(0), m_canvas(0) {
// TODO: Make sure number_plots != 0

    // Slice up the supplied view into the correct number of pieces.
    std::vector<double> domain;
    view->getDomain(domain);
    double width = domain.back() - domain.front();
    for (unsigned int ii = 0; ii < number_plots; ++ii) {
      double min = domain.front() + (ii / double(number_plots)) * width;
      double max = domain.front() + ((ii + 1.) / double(number_plots)) * width;
      SpectrumView * new_view = view->slice(min, max);
      trackView("StripChart", new_view);
    }
    display(main, w, h);
  }

  StripChart::StripChart(const view_cont_type & view, const TGWindow * main, UInt_t w, UInt_t h):
    m_view_cont(), m_hints_expand(kLHintsExpandX | kLHintsExpandY), m_main_frame(0),
    m_print(0), m_canvas(0) {
// TODO: Make sure number_plots != 0
    for (view_cont_type::const_iterator itor = view.begin(); itor != view.end(); ++itor) {
      m_view_cont.push_back((*itor)->clone());
    }
    display(main, w, h);
  }

  StripChart::~StripChart() {
    handleCloseWindow();
  }

  void StripChart::handlePrint() const {
    Gui::instance()->printCanvas(m_canvas);
  }

  void StripChart::handleCloseWindow() {
    while (!m_view_cont.empty()) {
      delete m_view_cont.back();
      m_view_cont.pop_back();
    }
    delete m_canvas; m_canvas = 0;
    delete m_print; m_print = 0;
    m_main_frame = 0;
  }

  void StripChart::trackView(const std::string & context, SpectrumView * view) {
    if (0 == view) throw std::runtime_error("Allocation of " + context + " failed");
    m_view_cont.push_back(view);
  }

  void StripChart::display(const TGWindow * main, UInt_t w, UInt_t h) {
    // TODO: Check pointers as they go.
    m_main_frame = new TGTransientFrame(gClient->GetRoot(), main, w, h);
    m_main_frame->SetWindowName("profit spectra strip chart");
    m_main_frame->Connect("CloseWindow()", "profit::StripChart", this, "handleCloseWindow()");

    m_print = new TGTextButton(m_main_frame, "Print");
    m_main_frame->AddFrame(m_print);
    m_print->Connect("Clicked()", "profit::StripChart", this, "handlePrint()");
    m_print->SetToolTipText("Print chart to file.");

    m_canvas = new TRootEmbeddedCanvas("Strip chart", m_main_frame, w, h - m_print->GetHeight());
    m_main_frame->AddFrame(m_canvas, &m_hints_expand);
    m_canvas->GetCanvas()->Divide(1, m_view_cont.size());
    m_canvas->GetCanvas()->SetFillStyle(4000);

    for (unsigned int ii = 0; ii < m_view_cont.size(); ++ii) {
      m_view_cont[ii]->display(m_canvas, kLHintsNormal, 200, 100, ii);
    }

    m_canvas->GetCanvas()->Update();

    m_main_frame->MapSubwindows();
    m_main_frame->Layout();
    m_main_frame->MapWindow();
    m_main_frame->CenterOnParent();

    gClient->WaitFor(m_main_frame);
  }

}
