/** \file TearOffMenu.cxx
    \brief Implementation of TearOffMenu class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/

#include "profit/Gui.h"
#include "profit/TearOffMenu.h"

#include "TGClient.h"
#include "TGFrame.h"
#include "TGMenu.h"
#include "TList.h"

#include <iostream>

ClassImp(profit::TearOffMenu)

namespace {

  // Special class derived from TGPopupMenu which delivers button clicks back to the Gui object.
  class TearOffPopup : public TGPopupMenu {
    public:
      TearOffPopup(TGCompositeFrame * mf, profit::TearOffMenu * tear_off_menu): TGPopupMenu(mf, 100, 100),
        m_tear_off_menu(tear_off_menu) { AddInput(kButtonPressMask | kButtonReleaseMask); }

      virtual Bool_t HandleButton(Event_t *) {
        m_tear_off_menu->handleMenu(GetCurrent()->GetEntryId());
        return kTRUE;
      }

    private:
      profit::TearOffMenu * m_tear_off_menu;
  };

}

namespace profit {

  TearOffMenu::prototype_cont_type TearOffMenu::s_prototype;

  TearOffMenu::TearOffMenu(const std::string & menu_id, Gui * gui, TGPopupMenu * popup_menu): m_menu_id(menu_id), m_tracker(),
    m_gui(gui), m_main_frame(0), m_popup_menu(0), m_handler_id(0) {
    m_main_frame = new TGMainFrame(gClient->GetRoot(), 100, 100);
    m_tracker.trackFrame("TearOffMenu main frame", m_main_frame, 0);
    m_main_frame->Connect("CloseWindow()", "profit::TearOffMenu", this, "handleCloseWindow()");

    m_popup_menu = new TearOffPopup(m_main_frame, this);
    m_tracker.trackFrame("TearOffMenu popup menu", m_popup_menu, m_main_frame, kLHintsExpandX | kLHintsExpandY);
    m_popup_menu->Connect("Activated(Int_t)", "profit::TearOffMenu", this, "handleMenu(int)");

    m_main_frame->SetWindowName(menu_id.c_str());
    copy(popup_menu);
//    m_main_frame->SetWindowSize(m_main_frame->GetDefaultWindowSize());
    
    m_main_frame->MapSubwindows();
    m_main_frame->Layout();
    m_main_frame->MapWindow();
  }

  TearOffMenu::~TearOffMenu() { s_prototype.erase(m_menu_id); }

  TearOffMenu * TearOffMenu::display(const std::string & menu_id, Gui * gui, TGPopupMenu * popup_menu, int handler_id) {
    prototype_cont_type::iterator itor = s_prototype.find(menu_id);
    if (s_prototype.end() == itor) {
      s_prototype.insert(prototype_cont_type::value_type(menu_id, new TearOffMenu(menu_id, gui, popup_menu)));
      itor = s_prototype.find(menu_id);
    } else {
      itor->second->raise();
    }
    itor->second->setHandler(handler_id);
    return itor->second;
  }

  void TearOffMenu::handleCloseWindow() { delete this; }

  void TearOffMenu::handleMenu(int id) {
    switch (m_handler_id) {
      case Gui::eFile:
        m_gui->handleMenu(id);
        break;
      case Gui::eSpectrum:
        m_gui->handleSpectrumMenu(id);
        break;
      default:
        break;
    }
  }

  TGPopupMenu * TearOffMenu::getMenu() { return m_popup_menu; }

  void TearOffMenu::copy(const TGPopupMenu * menu) {
    const TList * list = menu->GetListOfEntries();
    TIterator * itor = list->MakeIterator();
    TObject * obj = itor->Next();
    TGMenuEntry * entry;
    while (0 != (entry = dynamic_cast<TGMenuEntry *>(obj))) {
      const char * name = entry->GetName();
      if (0 == name || '\0' == *name) {
        getMenu()->AddSeparator();
      } else if (0 != std::string(name).find("---")) {
        getMenu()->AddEntry(name, entry->GetEntryId());
      } else {
        // Do nothing.
      }
      obj = itor->Next();
    }
  }

  void TearOffMenu::raise() { m_main_frame->RaiseWindow(); }

  void TearOffMenu::setHandler(int handler_id) { m_handler_id = handler_id; }

}
