/** \file TestAtomDb.cxx
    \brief Implementation of AtomDb class-specific parts of the unit test.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include <cstdio>

#include "profit/AtomDb.h"
#include "profit/Energy.h"
#include "profit/UnitTest.h"

namespace profit {

  void UnitTest::testAtomDb() {
    std::string method("testAtomDb()");

    // Create atomic database accessor object.
    info(method) << "Attempting to open atomic database file \"" << m_atom_db_file_name << "\"." << std::endl;
    AtomDb database(m_atom_db_file_name);
    info(method) << std::endl;

    // Get matching lines. There should be as many of those as there are records in the database, i.e. 34785.
    info(method) << "Attempting to get matching atomic lines immediately after creating database accessor." << std::endl;
    AtomDb::AtomicLineCont matches(database.getMatches());
    info(method) << std::endl;

    // Make sure there are the right number of matches.
    info(method) << "Confirming there are the right number of matching atomic lines before any filtering performed." << std::endl;
    if (34785 != matches.size())
      error(method) << "There were " << matches.size() << " lines in the atomic database, not 34785 as expected." << std::endl;
    info(method) << std::endl;

    // Spot-check the contents.
    size_t expected_index[] = { 0, 17392, 34784 };
    double expected_lambda[] = { 1.1982250E+00, 4.8695000E+01, 1.8207626E+07 };
    int expected_element[] = { 28, 14, 2 };
    int expected_ion[] = { 28, 10, 1 };
    int expected_upper[] = { 19, 51, 17 };
    int expected_lower[] = { 1, 2, 16 };
    double expected_peak[] = { 1.9437248E-20, 5.6163913E-18, 1.1728682E-20 };
    info(method) << "Spot-checking contents of several atomic database records." << std::endl;
    for (size_t index = 0; index < sizeof(expected_index) / sizeof(size_t); ++index) {
      size_t match_index = expected_index[index];
      if (!isEqual(matches[match_index].getLambda(), expected_lambda[index], 0., s_float_epsilon)) {
        error(method) << "In row " << match_index + 1 << ", value of lambda field was " <<
          matches[match_index].getLambda() << ", not " << expected_lambda[index] << " as expected." << std::endl;
      }
      if (matches[match_index].getElement() != expected_element[index]) {
        error(method) << "In row " << match_index + 1 << ", value of element field was " <<
          matches[match_index].getElement() << ", not " << expected_element[index] <<
          " as expected." << std::endl;
      }
      if (matches[match_index].getIon() != expected_ion[index]) {
        error(method) << "In row " << match_index + 1 << ", value of ion field was " <<
          matches[match_index].getIon() << ", not " << expected_ion[index] <<
          " as expected." << std::endl;
      }
      if (matches[match_index].getUpperLevel() != expected_upper[index]) {
        error(method) << "In row " << match_index + 1 << ", value of upper_lev field was " <<
          matches[match_index].getUpperLevel() << ", not " << expected_upper[index] <<
          " as expected." << std::endl;
      }
      if (matches[match_index].getLowerLevel() != expected_lower[index]) {
        error(method) << "In row " << match_index + 1 << ", value of lower_lev field was " <<
          matches[match_index].getLowerLevel() << ", not " << expected_lower[index] <<
          " as expected." << std::endl;
      }
      if (!isEqual(matches[match_index].getPeakEmissivity(), expected_peak[index], 0., s_float_epsilon)) {
        error(method) << "In row " << match_index + 1 << ", value of peakemissivity field was " <<
          matches[match_index].getPeakEmissivity() << ", not " << expected_peak[index] << " as expected." << std::endl;
      }
    }
    info(method) << std::endl;

    info(method) << "Checking format of Roman element name." << std::endl;
    std::string expected_string = "Ni XXVIII";
    if (matches[0].getRomanName() != expected_string)
      error(method) << "getRomanName() returned \"" << matches[0].getRomanName() << "\", not \"" <<
        expected_string << "\" as expected." << std::endl;
    info(method) << std::endl;

    info(method) << "Checking format of element label." << std::endl;
    expected_string = "10.3473 Ni XXVIII      19->1   1.000     ";
    if (matches[0].getLabel(matches[0], false) != expected_string)
      error(method) << "getLabel(matches[0], false) returned \"" << matches[0].getLabel(matches[0], false) << "\", not \"" <<
        expected_string << "\" as expected." << std::endl;
    expected_string = "1.1982  Ni XXVIII      19->1   1.00      ";
    if (matches[0].getLabel(matches[0], true) != expected_string)
      error(method) << "getLabel(matches[0], true) returned \"" << matches[0].getLabel(matches[0], true) << "\", not \"" <<
        expected_string << "\" as expected." << std::endl;
    info(method) << std::endl;

    info(method) << "Testing filtering of AtomDb object on emissivity." << std::endl;
    database.filter(1.e-15);
    info(method) << std::endl;

    info(method) << "Getting matches following filtering of AtomDb object on emissivity." << std::endl;
    matches = database.getMatches();
    info(method) << std::endl;

    info(method) << "Confirming matches are correct following filtering on emissivity." << std::endl;
    if (matches.size() != 1043)
      error(method) << "After database.filter(1.e-15), there are " << matches.size() <<
        " records, not 1043 as expected." << std::endl;
    info(method) << std::endl;

    info(method) << "Testing filtering of AtomDb object on energy." << std::endl;
    database.filter(Wavelength(1.e3), Wavelength(4.e1));
    info(method) << std::endl;

    info(method) << "Getting matches following filtering of AtomDb object on energy." << std::endl;
    matches = database.getMatches();
    if (matches.size() != 584)
      error(method) << "After database.filter(Wavelength(1.e4), Wavelength(4.e1)), there are " << matches.size() <<
        " records, not 584 as expected." << std::endl;
    info(method) << std::endl;

    info(method) << "Getting roman element of Si XIV" << std::endl;
    AtomicLine si_xiv("APED", 15., 14, 14, 1, 1, 1.);
    if (std::string::npos != si_xiv.getRomanName().find("Si XV"))
      error(method) << "Si XIV is being erroneously reported as Si XV." << std::endl;
    info(method) << std::endl;

    // Create atomic database accessor object for xstar.
    info(method) << "Attempting to open Xstar database file \"" << m_xstar_db_file_name << "\"." << std::endl;
    AtomDb xstar_db(m_xstar_db_file_name);
    info(method) << std::endl;

    // Filter energies.
    info(method) << "Attempting to filter Xstar database." << std::endl;
    matches = xstar_db.getMatches();
    info(method) << "Before filtering, number of matches is " << matches.size() << "." << std::endl;
    xstar_db.filter(Wavelength(1.e3), Wavelength(4.e1));
    matches = xstar_db.getMatches();
    info(method) << "After filtering, number of matches is " << matches.size() << "." << std::endl;
    info(method) << std::endl;
  }

}
