/** \file TestGui.cxx
    \brief Implementation of Gui class-specific parts of the unit test.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "TGListBox.h"

#include "profit/AtomDb.h"
#include "profit/Energy.h"
#include "profit/Gui.h"
#include "profit/SpectrumView.h"
#include "profit/Table.h"
#include "profit/UnitTestGui.h"

namespace profit {

  void UnitTestGui::testGui() {
    std::string method("testGui()");

    // Create Gui object to use for tests.
    info(method) << "Getting Gui singleton." << std::endl;
    Gui * gui(Gui::instance());
    if (0 == gui) {
      error(method) << "Failed to allocate or retrieve Gui." << std::endl;
      info(method) << std::endl;
      return;
    }
    info(method) << std::endl;

    // Before a spectrum is loaded, all button methods should be no-ops.
    info(method) << "Calling handleLineId() before spectrum loaded." << std::endl;
    gui->handleLineId();
    info(method) << std::endl;

    info(method) << "Calling handleVelocityView() before spectrum loaded." << std::endl;
    gui->handleVelocityView();
    info(method) << std::endl;

    info(method) << "Calling handleFitLine() before spectrum loaded." << std::endl;
    gui->handleFitLine();
    info(method) << std::endl;

    info(method) << "Calling handleStoreView() before spectrum loaded." << std::endl;
    gui->handleStoreView();
    info(method) << std::endl;

    info(method) << "Calling handleAtomicInfo() before spectrum loaded." << std::endl;
    gui->handleAtomicInfo();
    info(method) << std::endl;

    info(method) << "Calling handleLabelLines() before spectrum loaded." << std::endl;
    gui->handleLabelLines();
    info(method) << std::endl;

    info(method) << "Calling handleLabelPeaks() before spectrum loaded." << std::endl;
    gui->handleLabelPeaks();
    info(method) << std::endl;

    info(method) << "Calling handleClearLabels() before spectrum loaded." << std::endl;
    gui->handleClearLabels();
    info(method) << std::endl;

    info(method) << "Calling handleExpandX() before spectrum loaded." << std::endl;
    gui->handleExpandX();
    info(method) << std::endl;

    info(method) << "Calling handleStripChart() before spectrum loaded." << std::endl;
    gui->handleStripChart();
    info(method) << std::endl;

    info(method) << "Calling handleChartChecked() before spectrum loaded." << std::endl;
    gui->handleChartChecked();
    info(method) << std::endl;

    info(method) << "Calling handleSmooth() before spectrum loaded." << std::endl;
    gui->handleSmooth();
    info(method) << std::endl;

    // Test data import features.
    testDataImport();

    // Test file import features.
    testFileImport();

    // Load a spectrum into the GUI.
    info(method) << "Opening test spectrum " << m_spectrum_file_name << " and test response " << m_ebounds_file_name << "."
      << std::endl;
    gui->openSpecFile(m_spectrum_file_name, m_ebounds_file_name, false);
    info(method) << std::endl;

    // Perform a selection, simulating a user's zooming in on the main canvas.
    if (0 == gui->m_main_view) {
      error(method) << "After opening spectrum, Gui's m_main_view pointer is unexpectedly NULL." << std::endl;
    } else {
      info(method) << "Zooming in on Gui's current view." << std::endl;
      // Select a narrow band around an intriguing set of peaks.
      gui->m_main_view->select(Energy(1.82), Energy(1.88));
      info(method) << std::endl;

      if (0 != gui->m_line_box->GetNumberOfEntries())
        error(method) << "Number of entries in Gui::m_line_box is " << gui->m_line_box->GetNumberOfEntries() <<
          ", not 0 as expected before displaying line ids." << std::endl;
      if (0 != gui->m_matches.size())
        error(method) << "Number of entries in Gui::m_matches is " << gui->m_matches.size() <<
          ", not 0 as expected before displaying line ids." << std::endl;
      info(method) << std::endl;

      info(method) << "Testing handleFitLine on selected spectrum. This fit should converge." << std::endl;
      gui->handleFitLine();
      // TODO: find way of verifying fit succeeded.
      //if (0 == gui->m_fit_func)
      //  error(method) << "After what should have been a converged fit, gui->m_fit_func was 0." << std::endl;
      info(method) << std::endl;

      info(method) << "Storing this view in the \"save\" area." << std::endl;
      gui->handleStoreView();
      // For future reference, grab a copy of the stored view.
      SpectrumView * energy_view = gui->m_view_cont.back();
      info(method) << std::endl;

      info(method) << "Attempting to transform to velocity space before a line was selected. This should generate an error." <<
        std::endl;
      if (0 == gui->m_main_view || gui->m_main_view->isVelocity())
        error(method) << "Unexpected: before transforming to velocity space, main view is not a wavelength view." << std::endl;
      gui->handleVelocityView();
      if (0 == gui->m_main_view || gui->m_main_view->isVelocity())
        error(method) << "Unexpected: after transforming to velocity space, main view is not a wavelength view." << std::endl;
      info(method) << std::endl;

      info(method) << "Listing possible line identities in line list box." << std::endl;
      // Identify lines.
      gui->handleLineId();
      if (15 != gui->m_line_box->GetNumberOfEntries())
        error(method) << "After identifying lines in range [1.82, 1.88), there were " << gui->m_line_box->GetNumberOfEntries() <<
          " entries in Gui::m_line_box, not 15 as expected." << std::endl;
      if (15 != gui->m_matches.size())
        error(method) << "After identifying lines in range [1.82, 1.88), there were " << gui->m_matches.size() <<
          " entries in Gui::m_matches, not 15 as expected." << std::endl;
      info(method) << std::endl;

      // Select 2nd strongest line.
      info(method) << "Selecting second-strongest line in the list." << std::endl;
      gui->m_line_box->Select(1);
      if (1 != gui->m_line_box->GetSelected())
        error(method) << "After selecting the second line in the list, Gui::m_line_list->GetSelected() returned " <<
          gui->m_line_box->GetSelected() << ", not 1 as expected." << std::endl;
      info(method) << std::endl;

      // Convert view to velocity view.
      info(method) << "Converting to velocity view. The \"0\" should be at the position of the right-most peak." << std::endl;
      if (0 == gui->m_main_view || gui->m_main_view->isVelocity())
        error(method) << "Unexpected: before transforming to velocity space, main view is not a wavelength view." << std::endl;

      // Simulate clicking velocity button, by putting the button down, then calling handleVelocityView.
      gui->m_velocity_button->SetState(kButtonDown, kFALSE);
      gui->handleVelocityView();
      if (0 == gui->m_main_view || !gui->m_main_view->isVelocity())
        error(method) << "Unexpected: main view was not successfully transformed to velocity view." << std::endl;
      info(method) << std::endl;

      // The following test is to verify that Root does not generate a warning about replacing a histogram with the
      // same name.
      info(method) << "Converting to velocity view again without changing main view. This should not generate a Root warning."
        << std::endl;
      gui->handleVelocityView();
      info(method) << std::endl;

      info(method) << "Converting back from velocity view to wavelength view." << std::endl;
      if (0 == gui->m_main_view || !gui->m_main_view->isVelocity())
        error(method) << "Unexpected: before transforming back to wavelength space, main view is not a velocity view." << std::endl;

      // Simulate clicking wavelength button.
      gui->m_wavelength_button->Emit("Clicked");
      if (0 == gui->m_main_view || gui->m_main_view->isVelocity())
        error(method) << "Unexpected: main view was not successfully transformed back to wavelength view." << std::endl;
      info(method) << std::endl;

      info(method) << "Calling handleAtomicInfo() to display info about selected line." << std::endl;
      gui->handleAtomicInfo();
      info(method) << std::endl;

      info(method) << "Calling handleLabelLines() to display labels of lines on view." << std::endl;
      gui->handleLabelLines();
      info(method) << std::endl;

      info(method) << "Storing velocity view in the \"save\" area." << std::endl;
      gui->handleStoreView();
      info(method) << std::endl;

      info(method) << "Calling handleClearLabels() to remove labels from main view." << std::endl;
      gui->handleClearLabels();
      info(method) << std::endl;

      info(method) << "Calling handleExpandX() to zoom out slightly." << std::endl;
      double orig_min_energy = gui->m_main_view->getMinEnergy().energy();
      double orig_max_energy = gui->m_main_view->getMaxEnergy().energy();
      gui->handleExpandX();
      double min_energy = gui->m_main_view->getMinEnergy().energy();
      double max_energy = gui->m_main_view->getMaxEnergy().energy();
      if (min_energy >= orig_min_energy)
        error(method) << "After handleExpandX, minimum energy was " << min_energy <<
          ", which is not less than original minimum energy " << orig_min_energy << "." << std::endl;
      if (max_energy <= orig_max_energy)
        error(method) << "After handleExpandX, maximum energy was " << max_energy <<
          ", which is not greater than original maximum energy " << orig_max_energy << "." << std::endl;
      info(method) << std::endl;

      info(method) << "Calling handleStripChart()." << std::endl;
      gui->handleStripChart();
      info(method) << std::endl;

      info(method) << "Testing handleFitLine on selected velocity view. This fit should converge." << std::endl;
      gui->handleFitLine();
      // TODO: find way of verifying fit failed.
      //if (0 != gui->m_fit_func)
      //  error(method) << "After what should have been a bad fit, gui->m_fit_func was not 0." << std::endl;
      info(method) << std::endl;

      info(method) << "Restoring previously stored energy view to main view. Last fit result should be visible." << std::endl;
      gui->handleSelectView(*energy_view);
      info(method) << std::endl;

      info(method) << "Calling handleLabelLines()." << std::endl;
      gui->handleLabelLines();
      info(method) << std::endl;

      info(method) << "Calling handlePreferences()." << std::endl;
      gui->handlePreferences();
      info(method) << std::endl;

      info(method) << "Calling handleClearLabels()." << std::endl;
      gui->handleClearLabels();
      info(method) << std::endl;

      info(method) << "Calling handleLabelPeaks()." << std::endl;
      gui->handleLabelPeaks();
      info(method) << std::endl;

      info(method) << "Calling handleSmooth()." << std::endl;
      gui->handleSmooth();
      info(method) << std::endl;

      info(method) << "Calling handleAboutProfit()." << std::endl;
      gui->handleAboutProfit();
      info(method) << std::endl;

    }

  }

}
