/** \file TestTracker.cxx
    \brief Implementation of Tracker unit test.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#include "profit/Tracker.h"
#include "profit/UnitTest.h"

namespace profit {

  // Tests of the tracker class.
  void UnitTest::testTracker() {
    std::string method = "testTracker()";

    info(method) << "Creating tracker for tracking ints." << std::endl;
    Tracker<int> tracker;
    info(method) << std::endl;

    info(method) << "Looking for object named \"number\" in tracker before any objects tracked." << std::endl;
    Tracker<int>::const_iterator found = tracker.find("number");
    if (found != tracker.end()) {
      error(method) << "Found object \"number\" before it should exist" << std::endl;
    }
    info(method) << std::endl;

    info(method) << "Inserting \"number\" == 0." << std::endl;
    tracker.insert("number", 0);
    found = tracker.find("number");
    if (found == tracker.end()) {
      error(method) << "After inserting \"number\" == 0, Tracker::find did not find it." << std::endl;
    }
    info(method) << std::endl;

    info(method) << "Inserting \"number\" == 0 a second time." << std::endl;
    tracker.insert("number", 0);
    Tracker<int>::const_range_iterator range = tracker.equal_range("number");
    if (found != range.first) {
      error(method) << "After inserting \"number\" a second time, Tracker::equal_range did not find it." << std::endl;
    }
    ++found;
    if (tracker.end() != found) {
      error(method) << "After inserting \"number\" a second time, Tracker::equal_range found more than one match." << std::endl;
    }
    info(method) << std::endl;

    info(method) << "Inserting \"number\" == 1." << std::endl;
    tracker.insert("number", 1);
    found = tracker.find("number");
    if (found == tracker.end()) {
      error(method) << "After inserting \"number\" == 1, find did not find it." << std::endl;
    } else if (found->second != 0) {
      error(method) << "After inserting \"number\" == 1, find found element equal to " << found->second <<
        ", not 0 as expected." << std::endl;
    }
    info(method) << std::endl;
    
    info(method) << "Getting all integers which match \"number\"." << std::endl;
    range = tracker.equal_range("number");
    int expected_id = 0;
    for (Tracker<int>::const_iterator itor = range.first; itor != range.second; ++itor, ++expected_id) {
      if (itor->second != expected_id)
        error(method) << expected_id << "th match for \"number\" was " << itor->second << ", not " << expected_id <<
          ", as expected." << std::endl;
    }
    info(method) << std::endl;

    info(method) << "Removing \"number\" == 0." << std::endl;
    tracker.remove("number", 0);
    range = tracker.equal_range("number");
    found = range.first;
    if (found->second != 1)
      error(method) << "After removing \"number\" == 0, first match found by Tracker::equal_range was " << found->second <<
       ", not 1 as expected." << std::endl;
    ++found;
    if (tracker.end() != found)
      error(method) << "After removing \"number\" == 0, Tracker::equal_range found more than one match." << std::endl;
    info(method) << std::endl;
  }

}
